# This file was auto-generated by Fern from our API Definition.

from . import (
    account_base,
    accounts,
    audit,
    auth,
    auth_base,
    bridges,
    capabilities,
    capabilities_base,
    common,
    credentials,
    integration_base,
    integration_points,
    integrations,
    member_base,
    members,
    meta,
    operation_base,
    operations,
    organization,
    organization_base,
    organization_webhook_base,
    organization_webhook_events,
    organization_webhooks,
    permissions,
    permissionset,
    permissionset_base,
    providers_generated,
    role_base,
    roles,
    status,
    sub_orgs,
    token_base,
    tokens,
    transforms,
    usage,
)
from .account_base import AccountId
from .accounts import (
    Account,
    CreateAccountRequest,
    CreateAccountResponse,
    CreateAccountResponseResult,
    GetAccountResponse,
    ListAccountsResponse,
    PatchAccountResponse,
    UpdateAccountRequest,
    UpdateAccountResponse,
)
from .audit import Audit, AuditType, HttpMethod, ListAuditEventsResponse
from .auth import ChangePasswordRequest
from .auth_base import (
    AuthCode,
    ChangePasswordResponse,
    ChangePasswordResponseResult,
    LogonRequest,
    LogonResponse,
    LogonResponseResult,
)
from .bridges import (
    BridgeGroup,
    BridgeGroupId,
    BridgeLocalConfig,
    BridgeLocalStats,
    BridgeStatus,
    CreateBridgeRequest,
    CreateBridgeResponse,
    CreateBridgeResponseResult,
    GetBridgeResponse,
    GetBridgeStatusResponse,
    ListBridgesResponse,
    PatchBridgeResponse,
    UpdateBridgeRequest,
    UpdateBridgeResponse,
)
from .capabilities import (
    Connector,
    ConnectorOrId,
    FilterOperation,
    FilterType,
    GetProviderCapabilitiesExpandOptions,
    ListConnectorCapabilitiesExpandOptions,
    ListConnectorsCapabilitiesResponse,
    ListProviderCapabilitiesExpandOptions,
    ListProvidersCapabilitiesResponse,
    ProviderCapabilities,
    ProviderCapabilitiesResponse,
    ProviderFilter,
    ProviderOperations,
    RequestBody,
)
from .capabilities_base import CategoryId, ProviderId
from .common import (
    BadGatewayError,
    BadRequestError,
    Base,
    ConflictError,
    ErrorBody,
    ErrorParam,
    ForbiddenError,
    GatewayTimeoutError,
    Id,
    InternalServerError,
    MethodNotAllowedError,
    NotFoundError,
    NotImplementedError,
    PatchOp,
    PatchOperation,
    ServiceUnavailableError,
    TooManyRequestsError,
    UnauthorizedError,
    UnsupportedMediaTypeError,
)
from .credentials import (
    AwsCredential,
    AwsCredentialId,
    BasicCredential,
    BasicCredentialId,
    BridgeAwsCredential,
    BridgeBasicCredential,
    BridgeBasicCredentialId,
    BridgeCredential,
    BridgeCredential_BridgeAws,
    BridgeCredential_BridgeBasic,
    BridgeCredential_BridgeOAuthClient,
    BridgeCredential_BridgeSecret,
    BridgeCredential_BridgeToken,
    BridgeEnvironment,
    BridgeLiteral,
    BridgeLocalCredential,
    BridgeLocalCredential_Environment,
    BridgeLocalCredential_Literal,
    BridgeLocalCredential_Vault,
    BridgeOAuthClientCredential,
    BridgeOAuthClientCredentialId,
    BridgeSecretCredential,
    BridgeSecretCredentialId,
    BridgeTokenCredential,
    BridgeTokenCredentialId,
    BridgeType,
    CreateCredentialRequest,
    CreateCredentialResponse,
    Credential,
    CredentialBase,
    CredentialConfig,
    CredentialConfigNoSecret,
    CredentialConfig_Aws,
    CredentialConfig_Basic,
    CredentialConfig_Bridge,
    CredentialConfig_OAuthClient,
    CredentialConfig_Secret,
    CredentialConfig_Token,
    CredentialId,
    CredentialResponse,
    CredentialType,
    GetCredentialResponse,
    ListCredentialsResponse,
    LocalType,
    LookupCredentialResponse,
    ManagedType,
    OAuthClientCredential,
    OAuthClientCredentialId,
    OwnerType,
    PatchCredentialResponse,
    SecretCredential,
    SecretCredentialId,
    TokenCredential,
    TokenCredentialId,
    UpdateCredentialRequest,
    UpdateCredentialResponse,
    VaultCredential,
)
from .integration_base import IntegrationId
from .integration_points import (
    CreateIntegrationPointRequest,
    CreateIntegrationPointResponse,
    GetIntegrationPointResponse,
    IntegrationEnvironments,
    IntegrationPoint,
    IntegrationPointId,
    ListIntegrationPointsResponse,
    PatchIntegrationPointResponse,
    UpdateIntegrationPointRequest,
    UpdateIntegrationPointResponse,
)
from .integrations import (
    BridgeSelector,
    BridgeSelector_Id,
    BridgeSelector_Labels,
    CreateIntegrationRequest,
    CreateIntegrationResponse,
    CreateIntegrationResponseResult,
    GetIntegrationResponse,
    Integration,
    ListAccountIntegrationsResponse,
    ListIntegrationOptions,
    ListIntegrationsResponse,
    PatchIntegrationResponse,
    UpdateIntegrationRequest,
    UpdateIntegrationResponse,
    VerifyIntegrationRequest,
    WebhookConfig,
    WebhookEvent,
    WebhookItem,
)
from .member_base import (
    CreateMemberRequest,
    CreateMemberResponse,
    CreateMemberResponseResult,
    Member,
    MemberId,
    MemberOptions,
    Options,
    State,
)
from .members import (
    GetMemberResponse,
    ListMembersResponse,
    PatchMemberResponse,
    UpdateMemberRequest,
    UpdateMemberResponse,
)
from .meta import GetOpenApiSpecResponse
from .operation_base import Operation, OperationError, OperationId, OperationInput, OperationSchedule, OperationStatus
from .operations import ListOperationsResponse
from .organization import PatchOrganizationResponse, UpdateOrganizationRequest, UpdateOrganizationResponse
from .organization_base import (
    CreateOrganizationRequest,
    CreateOrganizationResponse,
    CreateOrganizationResponseResult,
    Environment,
    GetOrganizationResponse,
    ListOrganizationResponse,
    Organization,
    OrganizationId,
    OrganizationOptions,
    OrganizationType,
)
from .organization_webhook_base import WebhookFilter, WebhookId
from .organization_webhook_events import OrganizationWebhookPayload
from .organization_webhooks import (
    CreateOrganizationWebhookRequest,
    CreateOrganizationWebhookResponse,
    GetOrganizationWebhookResponse,
    ListOrganizationWebhooksResponse,
    OrganizationWebhook,
    OrganizationWebhookSecret,
    PatchOrganizationWebhookResponse,
    UpdateOrganizationWebhookRequest,
    UpdateOrganizationWebhookResponse,
)
from .permissions import Permission
from .permissionset import (
    AccountsActions,
    AccountsPermissions,
    AlarmPoliciesActions,
    AlarmPoliciesPermissions,
    AlarmsActions,
    AlarmsPermissions,
    ApiPermissionMap,
    AuditActions,
    AuditPermissions,
    AuthActions,
    AuthPermissions,
    BridgesActions,
    BridgesPermissions,
    CredentialsActions,
    CredentialsPermissions,
    GetPermissionSetResponse,
    IntegrationPointsActions,
    IntegrationPointsPermissions,
    IntegrationsActions,
    IntegrationsPermissions,
    ListPermissionSetsResponse,
    MembersActions,
    MembersPermissions,
    OperationsActions,
    OperationsPermissions,
    OrganizationActions,
    OrganizationPermissions,
    PermissionSet,
    PermissionSetActions,
    PermissionSetPermissions,
    ReadWriteActions,
    ReadWritePermissions,
    ResourceRestrictions,
    RolesActions,
    RolesPermissions,
    StatusActions,
    StatusPermissions,
    SubOrgsActions,
    SubOrgsPermissions,
    TokensActions,
    TokensPermissions,
    TransformsActions,
    TransformsPermissions,
    WebhooksActions,
    WebhooksPermissions,
)
from .permissionset_base import Permissions
from .providers_generated import (
    ArmisCredential,
    ArmisCredential_Token,
    ArmisCredential_TokenId,
    AssetsArmisCentrix,
    AssetsNozomiVantage,
    AssetsServiceNow,
    AwsS3Credential,
    AwsS3Credential_Aws,
    AwsS3Credential_AwsId,
    AwsSecurityLakeCredential,
    AwsSecurityLakeCredential_Aws,
    AwsSecurityLakeCredential_AwsId,
    AwsSqsCredential,
    AwsSqsCredential_Aws,
    AwsSqsCredential_AwsId,
    AzureBlobCredential,
    AzureBlobCredential_Token,
    AzureBlobCredential_TokenId,
    AzureMonitorLogsCredential,
    AzureMonitorLogsCredential_Token,
    AzureMonitorLogsCredential_TokenId,
    CrowdStrikeCredential,
    CrowdStrikeCredential_OAuthClient,
    CrowdStrikeCredential_OAuthClientId,
    CrowdstrikeHecCredential,
    CrowdstrikeHecCredential_Token,
    CrowdstrikeHecCredential_TokenId,
    CustomFieldMapping,
    DefenderCredential,
    DefenderCredential_OAuthClient,
    DefenderCredential_OAuthClientId,
    EdrCrowdStrike,
    EdrDefender,
    EdrSentinelOne,
    EdrSophos,
    ElasticsearchAuthOptions,
    ElasticsearchBridgeCredentials,
    ElasticsearchBridgeCredentials_BridgeBasic,
    ElasticsearchBridgeCredentials_BridgeBasicId,
    ElasticsearchBridgeCredentials_BridgeOAuthClient,
    ElasticsearchBridgeCredentials_BridgeOAuthClientId,
    ElasticsearchBridgeCredentials_BridgeToken,
    ElasticsearchBridgeCredentials_BridgeTokenId,
    ElasticsearchBridgeSharedSecret,
    ElasticsearchBridgeSharedSecret_BridgeSecret,
    ElasticsearchBridgeSharedSecret_BridgeSecretId,
    ElasticsearchCredential,
    ElasticsearchCredential_Basic,
    ElasticsearchCredential_BasicId,
    ElasticsearchCredential_Bridge,
    ElasticsearchCredential_OAuthClient,
    ElasticsearchCredential_OAuthClientId,
    ElasticsearchCredential_Token,
    ElasticsearchCredential_TokenId,
    ElasticsearchSharedSecret,
    ElasticsearchSharedSecret_Bridge,
    ElasticsearchSharedSecret_Secret,
    ElasticsearchSharedSecret_SecretId,
    EntraIdCredential,
    EntraIdCredential_OAuthClient,
    EntraIdCredential_OAuthClientId,
    GcsCredential,
    GcsCredential_Aws,
    GcsCredential_AwsId,
    GoogleChronicleCredential,
    GoogleChronicleCredential_OAuthClient,
    GoogleChronicleCredential_OAuthClientId,
    GoogleCredential,
    GoogleCredential_OAuthClient,
    GoogleCredential_OAuthClientId,
    IdentityEntraId,
    IdentityGoogle,
    IdentityOkta,
    IdentityPingOne,
    JiraCredential,
    JiraCredential_Basic,
    JiraCredential_BasicId,
    MockAssetsServiceNow,
    NotificationsJira,
    NotificationsMock,
    NotificationsSlack,
    NotificationsTeams,
    NozomiVantageCredential,
    NozomiVantageCredential_Basic,
    NozomiVantageCredential_BasicId,
    NucleusCredential,
    NucleusCredential_Token,
    NucleusCredential_TokenId,
    OktaCredential,
    OktaCredential_OAuthClient,
    OktaCredential_OAuthClientId,
    OktaCredential_Token,
    OktaCredential_TokenId,
    PagerDutyCredential,
    PagerDutyCredential_Token,
    PagerDutyCredential_TokenId,
    PingOneCredential,
    PingOneCredential_Token,
    PingOneCredential_TokenId,
    ProviderConfig,
    ProviderConfigId,
    ProviderConfig_AssetsArmisCentrix,
    ProviderConfig_AssetsNozomiVantage,
    ProviderConfig_AssetsServicenow,
    ProviderConfig_EdrCrowdstrike,
    ProviderConfig_EdrDefender,
    ProviderConfig_EdrSentinelone,
    ProviderConfig_EdrSophos,
    ProviderConfig_IdentityEntraId,
    ProviderConfig_IdentityGoogle,
    ProviderConfig_IdentityOkta,
    ProviderConfig_IdentityPingone,
    ProviderConfig_MockAssetsServicenow,
    ProviderConfig_NotificationsJira,
    ProviderConfig_NotificationsMockNotifications,
    ProviderConfig_NotificationsSlack,
    ProviderConfig_NotificationsTeams,
    ProviderConfig_SiemElasticsearch,
    ProviderConfig_SiemGoogleChronicle,
    ProviderConfig_SiemMockSiem,
    ProviderConfig_SiemQRadar,
    ProviderConfig_SiemRapid7Insightidr,
    ProviderConfig_SiemSentinel,
    ProviderConfig_SiemSplunk,
    ProviderConfig_SiemSumoLogic,
    ProviderConfig_SinkAwsSecurityLake,
    ProviderConfig_SinkAwsSqs,
    ProviderConfig_SinkAzureMonitorLogs,
    ProviderConfig_SinkCrowdstrikeHec,
    ProviderConfig_SinkMockSink,
    ProviderConfig_StorageAwsS3,
    ProviderConfig_StorageAzureBlob,
    ProviderConfig_StorageGcs,
    ProviderConfig_StorageMockStorage,
    ProviderConfig_TicketingJira,
    ProviderConfig_TicketingMockTicketing,
    ProviderConfig_TicketingPagerduty,
    ProviderConfig_TicketingServicenow,
    ProviderConfig_TicketingTorq,
    ProviderConfig_VulnerabilitiesCrowdstrike,
    ProviderConfig_VulnerabilitiesNucleus,
    ProviderConfig_VulnerabilitiesQualysCloud,
    ProviderConfig_VulnerabilitiesRapid7InsightCloud,
    ProviderConfig_VulnerabilitiesTaniumCloud,
    ProviderConfig_VulnerabilitiesTenableCloud,
    QRadarCredential,
    QRadarCredential_Token,
    QRadarCredential_TokenId,
    QualysCloudCredential,
    QualysCloudCredential_Basic,
    QualysCloudCredential_BasicId,
    Rapid7InsightCloudCredential,
    Rapid7InsightCloudCredential_Token,
    Rapid7InsightCloudCredential_TokenId,
    SentinelCredential,
    SentinelCredential_OAuthClient,
    SentinelCredential_OAuthClientId,
    SentinelOneCredential,
    SentinelOneCredential_Token,
    SentinelOneCredential_TokenId,
    ServiceNowCredential,
    ServiceNowCredential_Basic,
    ServiceNowCredential_BasicId,
    ServiceNowCredential_Token,
    ServiceNowCredential_TokenId,
    SiemElasticsearch,
    SiemGoogleChronicle,
    SiemMock,
    SiemQRadar,
    SiemRapid7InsightIdr,
    SiemSentinel,
    SiemSplunk,
    SiemSumoLogic,
    SinkAwsSecurityLake,
    SinkAwsSqs,
    SinkAzureMonitorLogs,
    SinkCrowdstrikeHec,
    SinkMock,
    SlackCredential,
    SlackCredential_Token,
    SlackCredential_TokenId,
    SophosCredential,
    SophosCredential_OAuthClient,
    SophosCredential_OAuthClientId,
    SplunkBridgeHecToken,
    SplunkBridgeHecToken_BridgeToken,
    SplunkBridgeHecToken_BridgeTokenId,
    SplunkBridgeSearchCredential,
    SplunkBridgeSearchCredential_BridgeToken,
    SplunkBridgeSearchCredential_BridgeTokenId,
    SplunkHecToken,
    SplunkHecToken_Bridge,
    SplunkHecToken_Token,
    SplunkHecToken_TokenId,
    SplunkSearchCredential,
    SplunkSearchCredential_Bridge,
    SplunkSearchCredential_Token,
    SplunkSearchCredential_TokenId,
    StorageAwsS3,
    StorageAzureBlob,
    StorageGcs,
    StorageMock,
    SumoLogicCollectionUrl,
    SumoLogicCollectionUrl_Secret,
    SumoLogicCollectionUrl_SecretId,
    SumoLogicCredential,
    SumoLogicCredential_Basic,
    SumoLogicCredential_BasicId,
    TaniumCloudCredential,
    TaniumCloudCredential_Token,
    TaniumCloudCredential_TokenId,
    TeamsCredential,
    TeamsCredential_OAuthClient,
    TeamsCredential_OAuthClientId,
    TeamsCredential_WebhookUrl,
    TeamsCredential_WebhookUrlId,
    TenableCloudCredential,
    TenableCloudCredential_Token,
    TenableCloudCredential_TokenId,
    TicketingJira,
    TicketingMock,
    TicketingPagerDuty,
    TicketingServiceNow,
    TicketingTorq,
    TorqCredential,
    TorqCredential_OAuthClient,
    TorqCredential_OAuthClientId,
    VulnerabilitiesCrowdStrike,
    VulnerabilitiesNucleus,
    VulnerabilitiesQualysCloud,
    VulnerabilitiesRapid7InsightCloud,
    VulnerabilitiesTaniumCloud,
    VulnerabilitiesTenableCloud,
)
from .role_base import AdhocRole, Resources, RoleAccounts, RoleId, RoleIntegrations, RoleName, RoleOrganizations
from .roles import (
    BuiltinRoles,
    CreateRoleRequest,
    CreateRoleResponse,
    GetRoleResponse,
    ListRolesResponse,
    PatchRoleResponse,
    RoleDefinition,
    UpdateRoleRequest,
    UpdateRoleResponse,
)
from .status import (
    GetIntegrationTimeseries,
    GetIntegrationTimeseriesResult,
    GetStatusResponse,
    GetStatusTimeseries,
    GetStatusTimeseriesResult,
    ListStatusEventsResponse,
    ListStatusOptions,
    ListStatusResponse,
    Status,
    StatusEvent,
    TimeseriesOptions,
    TimeseriesResult,
)
from .token_base import Token, TokenId, TokenOwnerType, TokenPair
from .tokens import (
    CreateIntegrationTokenRequest,
    CreateIntegrationTokenResponse,
    CreateTokenRequest,
    CreateTokenResponse,
    GetTokenResponse,
    ListTokensResponse,
    RefreshToken,
    RefreshTokenResponse,
    ResetTokenResponse,
)
from .transforms import (
    CreateTransformRequest,
    CreateTransformResponse,
    GetTransformResponse,
    ListTransformsResponse,
    PatchTransformResponse,
    Transform,
    TransformId,
    UpdateTransformRequest,
    UpdateTransformResponse,
)
from .usage import Usage

__all__ = [
    "Account",
    "AccountId",
    "AccountsActions",
    "AccountsPermissions",
    "AdhocRole",
    "AlarmPoliciesActions",
    "AlarmPoliciesPermissions",
    "AlarmsActions",
    "AlarmsPermissions",
    "ApiPermissionMap",
    "ArmisCredential",
    "ArmisCredential_Token",
    "ArmisCredential_TokenId",
    "AssetsArmisCentrix",
    "AssetsNozomiVantage",
    "AssetsServiceNow",
    "Audit",
    "AuditActions",
    "AuditPermissions",
    "AuditType",
    "AuthActions",
    "AuthCode",
    "AuthPermissions",
    "AwsCredential",
    "AwsCredentialId",
    "AwsS3Credential",
    "AwsS3Credential_Aws",
    "AwsS3Credential_AwsId",
    "AwsSecurityLakeCredential",
    "AwsSecurityLakeCredential_Aws",
    "AwsSecurityLakeCredential_AwsId",
    "AwsSqsCredential",
    "AwsSqsCredential_Aws",
    "AwsSqsCredential_AwsId",
    "AzureBlobCredential",
    "AzureBlobCredential_Token",
    "AzureBlobCredential_TokenId",
    "AzureMonitorLogsCredential",
    "AzureMonitorLogsCredential_Token",
    "AzureMonitorLogsCredential_TokenId",
    "BadGatewayError",
    "BadRequestError",
    "Base",
    "BasicCredential",
    "BasicCredentialId",
    "BridgeAwsCredential",
    "BridgeBasicCredential",
    "BridgeBasicCredentialId",
    "BridgeCredential",
    "BridgeCredential_BridgeAws",
    "BridgeCredential_BridgeBasic",
    "BridgeCredential_BridgeOAuthClient",
    "BridgeCredential_BridgeSecret",
    "BridgeCredential_BridgeToken",
    "BridgeEnvironment",
    "BridgeGroup",
    "BridgeGroupId",
    "BridgeLiteral",
    "BridgeLocalConfig",
    "BridgeLocalCredential",
    "BridgeLocalCredential_Environment",
    "BridgeLocalCredential_Literal",
    "BridgeLocalCredential_Vault",
    "BridgeLocalStats",
    "BridgeOAuthClientCredential",
    "BridgeOAuthClientCredentialId",
    "BridgeSecretCredential",
    "BridgeSecretCredentialId",
    "BridgeSelector",
    "BridgeSelector_Id",
    "BridgeSelector_Labels",
    "BridgeStatus",
    "BridgeTokenCredential",
    "BridgeTokenCredentialId",
    "BridgeType",
    "BridgesActions",
    "BridgesPermissions",
    "BuiltinRoles",
    "CategoryId",
    "ChangePasswordRequest",
    "ChangePasswordResponse",
    "ChangePasswordResponseResult",
    "ConflictError",
    "Connector",
    "ConnectorOrId",
    "CreateAccountRequest",
    "CreateAccountResponse",
    "CreateAccountResponseResult",
    "CreateBridgeRequest",
    "CreateBridgeResponse",
    "CreateBridgeResponseResult",
    "CreateCredentialRequest",
    "CreateCredentialResponse",
    "CreateIntegrationPointRequest",
    "CreateIntegrationPointResponse",
    "CreateIntegrationRequest",
    "CreateIntegrationResponse",
    "CreateIntegrationResponseResult",
    "CreateIntegrationTokenRequest",
    "CreateIntegrationTokenResponse",
    "CreateMemberRequest",
    "CreateMemberResponse",
    "CreateMemberResponseResult",
    "CreateOrganizationRequest",
    "CreateOrganizationResponse",
    "CreateOrganizationResponseResult",
    "CreateOrganizationWebhookRequest",
    "CreateOrganizationWebhookResponse",
    "CreateRoleRequest",
    "CreateRoleResponse",
    "CreateTokenRequest",
    "CreateTokenResponse",
    "CreateTransformRequest",
    "CreateTransformResponse",
    "Credential",
    "CredentialBase",
    "CredentialConfig",
    "CredentialConfigNoSecret",
    "CredentialConfig_Aws",
    "CredentialConfig_Basic",
    "CredentialConfig_Bridge",
    "CredentialConfig_OAuthClient",
    "CredentialConfig_Secret",
    "CredentialConfig_Token",
    "CredentialId",
    "CredentialResponse",
    "CredentialType",
    "CredentialsActions",
    "CredentialsPermissions",
    "CrowdStrikeCredential",
    "CrowdStrikeCredential_OAuthClient",
    "CrowdStrikeCredential_OAuthClientId",
    "CrowdstrikeHecCredential",
    "CrowdstrikeHecCredential_Token",
    "CrowdstrikeHecCredential_TokenId",
    "CustomFieldMapping",
    "DefenderCredential",
    "DefenderCredential_OAuthClient",
    "DefenderCredential_OAuthClientId",
    "EdrCrowdStrike",
    "EdrDefender",
    "EdrSentinelOne",
    "EdrSophos",
    "ElasticsearchAuthOptions",
    "ElasticsearchBridgeCredentials",
    "ElasticsearchBridgeCredentials_BridgeBasic",
    "ElasticsearchBridgeCredentials_BridgeBasicId",
    "ElasticsearchBridgeCredentials_BridgeOAuthClient",
    "ElasticsearchBridgeCredentials_BridgeOAuthClientId",
    "ElasticsearchBridgeCredentials_BridgeToken",
    "ElasticsearchBridgeCredentials_BridgeTokenId",
    "ElasticsearchBridgeSharedSecret",
    "ElasticsearchBridgeSharedSecret_BridgeSecret",
    "ElasticsearchBridgeSharedSecret_BridgeSecretId",
    "ElasticsearchCredential",
    "ElasticsearchCredential_Basic",
    "ElasticsearchCredential_BasicId",
    "ElasticsearchCredential_Bridge",
    "ElasticsearchCredential_OAuthClient",
    "ElasticsearchCredential_OAuthClientId",
    "ElasticsearchCredential_Token",
    "ElasticsearchCredential_TokenId",
    "ElasticsearchSharedSecret",
    "ElasticsearchSharedSecret_Bridge",
    "ElasticsearchSharedSecret_Secret",
    "ElasticsearchSharedSecret_SecretId",
    "EntraIdCredential",
    "EntraIdCredential_OAuthClient",
    "EntraIdCredential_OAuthClientId",
    "Environment",
    "ErrorBody",
    "ErrorParam",
    "FilterOperation",
    "FilterType",
    "ForbiddenError",
    "GatewayTimeoutError",
    "GcsCredential",
    "GcsCredential_Aws",
    "GcsCredential_AwsId",
    "GetAccountResponse",
    "GetBridgeResponse",
    "GetBridgeStatusResponse",
    "GetCredentialResponse",
    "GetIntegrationPointResponse",
    "GetIntegrationResponse",
    "GetIntegrationTimeseries",
    "GetIntegrationTimeseriesResult",
    "GetMemberResponse",
    "GetOpenApiSpecResponse",
    "GetOrganizationResponse",
    "GetOrganizationWebhookResponse",
    "GetPermissionSetResponse",
    "GetProviderCapabilitiesExpandOptions",
    "GetRoleResponse",
    "GetStatusResponse",
    "GetStatusTimeseries",
    "GetStatusTimeseriesResult",
    "GetTokenResponse",
    "GetTransformResponse",
    "GoogleChronicleCredential",
    "GoogleChronicleCredential_OAuthClient",
    "GoogleChronicleCredential_OAuthClientId",
    "GoogleCredential",
    "GoogleCredential_OAuthClient",
    "GoogleCredential_OAuthClientId",
    "HttpMethod",
    "Id",
    "IdentityEntraId",
    "IdentityGoogle",
    "IdentityOkta",
    "IdentityPingOne",
    "Integration",
    "IntegrationEnvironments",
    "IntegrationId",
    "IntegrationPoint",
    "IntegrationPointId",
    "IntegrationPointsActions",
    "IntegrationPointsPermissions",
    "IntegrationsActions",
    "IntegrationsPermissions",
    "InternalServerError",
    "JiraCredential",
    "JiraCredential_Basic",
    "JiraCredential_BasicId",
    "ListAccountIntegrationsResponse",
    "ListAccountsResponse",
    "ListAuditEventsResponse",
    "ListBridgesResponse",
    "ListConnectorCapabilitiesExpandOptions",
    "ListConnectorsCapabilitiesResponse",
    "ListCredentialsResponse",
    "ListIntegrationOptions",
    "ListIntegrationPointsResponse",
    "ListIntegrationsResponse",
    "ListMembersResponse",
    "ListOperationsResponse",
    "ListOrganizationResponse",
    "ListOrganizationWebhooksResponse",
    "ListPermissionSetsResponse",
    "ListProviderCapabilitiesExpandOptions",
    "ListProvidersCapabilitiesResponse",
    "ListRolesResponse",
    "ListStatusEventsResponse",
    "ListStatusOptions",
    "ListStatusResponse",
    "ListTokensResponse",
    "ListTransformsResponse",
    "LocalType",
    "LogonRequest",
    "LogonResponse",
    "LogonResponseResult",
    "LookupCredentialResponse",
    "ManagedType",
    "Member",
    "MemberId",
    "MemberOptions",
    "MembersActions",
    "MembersPermissions",
    "MethodNotAllowedError",
    "MockAssetsServiceNow",
    "NotFoundError",
    "NotImplementedError",
    "NotificationsJira",
    "NotificationsMock",
    "NotificationsSlack",
    "NotificationsTeams",
    "NozomiVantageCredential",
    "NozomiVantageCredential_Basic",
    "NozomiVantageCredential_BasicId",
    "NucleusCredential",
    "NucleusCredential_Token",
    "NucleusCredential_TokenId",
    "OAuthClientCredential",
    "OAuthClientCredentialId",
    "OktaCredential",
    "OktaCredential_OAuthClient",
    "OktaCredential_OAuthClientId",
    "OktaCredential_Token",
    "OktaCredential_TokenId",
    "Operation",
    "OperationError",
    "OperationId",
    "OperationInput",
    "OperationSchedule",
    "OperationStatus",
    "OperationsActions",
    "OperationsPermissions",
    "Options",
    "Organization",
    "OrganizationActions",
    "OrganizationId",
    "OrganizationOptions",
    "OrganizationPermissions",
    "OrganizationType",
    "OrganizationWebhook",
    "OrganizationWebhookPayload",
    "OrganizationWebhookSecret",
    "OwnerType",
    "PagerDutyCredential",
    "PagerDutyCredential_Token",
    "PagerDutyCredential_TokenId",
    "PatchAccountResponse",
    "PatchBridgeResponse",
    "PatchCredentialResponse",
    "PatchIntegrationPointResponse",
    "PatchIntegrationResponse",
    "PatchMemberResponse",
    "PatchOp",
    "PatchOperation",
    "PatchOrganizationResponse",
    "PatchOrganizationWebhookResponse",
    "PatchRoleResponse",
    "PatchTransformResponse",
    "Permission",
    "PermissionSet",
    "PermissionSetActions",
    "PermissionSetPermissions",
    "Permissions",
    "PingOneCredential",
    "PingOneCredential_Token",
    "PingOneCredential_TokenId",
    "ProviderCapabilities",
    "ProviderCapabilitiesResponse",
    "ProviderConfig",
    "ProviderConfigId",
    "ProviderConfig_AssetsArmisCentrix",
    "ProviderConfig_AssetsNozomiVantage",
    "ProviderConfig_AssetsServicenow",
    "ProviderConfig_EdrCrowdstrike",
    "ProviderConfig_EdrDefender",
    "ProviderConfig_EdrSentinelone",
    "ProviderConfig_EdrSophos",
    "ProviderConfig_IdentityEntraId",
    "ProviderConfig_IdentityGoogle",
    "ProviderConfig_IdentityOkta",
    "ProviderConfig_IdentityPingone",
    "ProviderConfig_MockAssetsServicenow",
    "ProviderConfig_NotificationsJira",
    "ProviderConfig_NotificationsMockNotifications",
    "ProviderConfig_NotificationsSlack",
    "ProviderConfig_NotificationsTeams",
    "ProviderConfig_SiemElasticsearch",
    "ProviderConfig_SiemGoogleChronicle",
    "ProviderConfig_SiemMockSiem",
    "ProviderConfig_SiemQRadar",
    "ProviderConfig_SiemRapid7Insightidr",
    "ProviderConfig_SiemSentinel",
    "ProviderConfig_SiemSplunk",
    "ProviderConfig_SiemSumoLogic",
    "ProviderConfig_SinkAwsSecurityLake",
    "ProviderConfig_SinkAwsSqs",
    "ProviderConfig_SinkAzureMonitorLogs",
    "ProviderConfig_SinkCrowdstrikeHec",
    "ProviderConfig_SinkMockSink",
    "ProviderConfig_StorageAwsS3",
    "ProviderConfig_StorageAzureBlob",
    "ProviderConfig_StorageGcs",
    "ProviderConfig_StorageMockStorage",
    "ProviderConfig_TicketingJira",
    "ProviderConfig_TicketingMockTicketing",
    "ProviderConfig_TicketingPagerduty",
    "ProviderConfig_TicketingServicenow",
    "ProviderConfig_TicketingTorq",
    "ProviderConfig_VulnerabilitiesCrowdstrike",
    "ProviderConfig_VulnerabilitiesNucleus",
    "ProviderConfig_VulnerabilitiesQualysCloud",
    "ProviderConfig_VulnerabilitiesRapid7InsightCloud",
    "ProviderConfig_VulnerabilitiesTaniumCloud",
    "ProviderConfig_VulnerabilitiesTenableCloud",
    "ProviderFilter",
    "ProviderId",
    "ProviderOperations",
    "QRadarCredential",
    "QRadarCredential_Token",
    "QRadarCredential_TokenId",
    "QualysCloudCredential",
    "QualysCloudCredential_Basic",
    "QualysCloudCredential_BasicId",
    "Rapid7InsightCloudCredential",
    "Rapid7InsightCloudCredential_Token",
    "Rapid7InsightCloudCredential_TokenId",
    "ReadWriteActions",
    "ReadWritePermissions",
    "RefreshToken",
    "RefreshTokenResponse",
    "RequestBody",
    "ResetTokenResponse",
    "ResourceRestrictions",
    "Resources",
    "RoleAccounts",
    "RoleDefinition",
    "RoleId",
    "RoleIntegrations",
    "RoleName",
    "RoleOrganizations",
    "RolesActions",
    "RolesPermissions",
    "SecretCredential",
    "SecretCredentialId",
    "SentinelCredential",
    "SentinelCredential_OAuthClient",
    "SentinelCredential_OAuthClientId",
    "SentinelOneCredential",
    "SentinelOneCredential_Token",
    "SentinelOneCredential_TokenId",
    "ServiceNowCredential",
    "ServiceNowCredential_Basic",
    "ServiceNowCredential_BasicId",
    "ServiceNowCredential_Token",
    "ServiceNowCredential_TokenId",
    "ServiceUnavailableError",
    "SiemElasticsearch",
    "SiemGoogleChronicle",
    "SiemMock",
    "SiemQRadar",
    "SiemRapid7InsightIdr",
    "SiemSentinel",
    "SiemSplunk",
    "SiemSumoLogic",
    "SinkAwsSecurityLake",
    "SinkAwsSqs",
    "SinkAzureMonitorLogs",
    "SinkCrowdstrikeHec",
    "SinkMock",
    "SlackCredential",
    "SlackCredential_Token",
    "SlackCredential_TokenId",
    "SophosCredential",
    "SophosCredential_OAuthClient",
    "SophosCredential_OAuthClientId",
    "SplunkBridgeHecToken",
    "SplunkBridgeHecToken_BridgeToken",
    "SplunkBridgeHecToken_BridgeTokenId",
    "SplunkBridgeSearchCredential",
    "SplunkBridgeSearchCredential_BridgeToken",
    "SplunkBridgeSearchCredential_BridgeTokenId",
    "SplunkHecToken",
    "SplunkHecToken_Bridge",
    "SplunkHecToken_Token",
    "SplunkHecToken_TokenId",
    "SplunkSearchCredential",
    "SplunkSearchCredential_Bridge",
    "SplunkSearchCredential_Token",
    "SplunkSearchCredential_TokenId",
    "State",
    "Status",
    "StatusActions",
    "StatusEvent",
    "StatusPermissions",
    "StorageAwsS3",
    "StorageAzureBlob",
    "StorageGcs",
    "StorageMock",
    "SubOrgsActions",
    "SubOrgsPermissions",
    "SumoLogicCollectionUrl",
    "SumoLogicCollectionUrl_Secret",
    "SumoLogicCollectionUrl_SecretId",
    "SumoLogicCredential",
    "SumoLogicCredential_Basic",
    "SumoLogicCredential_BasicId",
    "TaniumCloudCredential",
    "TaniumCloudCredential_Token",
    "TaniumCloudCredential_TokenId",
    "TeamsCredential",
    "TeamsCredential_OAuthClient",
    "TeamsCredential_OAuthClientId",
    "TeamsCredential_WebhookUrl",
    "TeamsCredential_WebhookUrlId",
    "TenableCloudCredential",
    "TenableCloudCredential_Token",
    "TenableCloudCredential_TokenId",
    "TicketingJira",
    "TicketingMock",
    "TicketingPagerDuty",
    "TicketingServiceNow",
    "TicketingTorq",
    "TimeseriesOptions",
    "TimeseriesResult",
    "Token",
    "TokenCredential",
    "TokenCredentialId",
    "TokenId",
    "TokenOwnerType",
    "TokenPair",
    "TokensActions",
    "TokensPermissions",
    "TooManyRequestsError",
    "TorqCredential",
    "TorqCredential_OAuthClient",
    "TorqCredential_OAuthClientId",
    "Transform",
    "TransformId",
    "TransformsActions",
    "TransformsPermissions",
    "UnauthorizedError",
    "UnsupportedMediaTypeError",
    "UpdateAccountRequest",
    "UpdateAccountResponse",
    "UpdateBridgeRequest",
    "UpdateBridgeResponse",
    "UpdateCredentialRequest",
    "UpdateCredentialResponse",
    "UpdateIntegrationPointRequest",
    "UpdateIntegrationPointResponse",
    "UpdateIntegrationRequest",
    "UpdateIntegrationResponse",
    "UpdateMemberRequest",
    "UpdateMemberResponse",
    "UpdateOrganizationRequest",
    "UpdateOrganizationResponse",
    "UpdateOrganizationWebhookRequest",
    "UpdateOrganizationWebhookResponse",
    "UpdateRoleRequest",
    "UpdateRoleResponse",
    "UpdateTransformRequest",
    "UpdateTransformResponse",
    "Usage",
    "VaultCredential",
    "VerifyIntegrationRequest",
    "VulnerabilitiesCrowdStrike",
    "VulnerabilitiesNucleus",
    "VulnerabilitiesQualysCloud",
    "VulnerabilitiesRapid7InsightCloud",
    "VulnerabilitiesTaniumCloud",
    "VulnerabilitiesTenableCloud",
    "WebhookConfig",
    "WebhookEvent",
    "WebhookFilter",
    "WebhookId",
    "WebhookItem",
    "WebhooksActions",
    "WebhooksPermissions",
    "account_base",
    "accounts",
    "audit",
    "auth",
    "auth_base",
    "bridges",
    "capabilities",
    "capabilities_base",
    "common",
    "credentials",
    "integration_base",
    "integration_points",
    "integrations",
    "member_base",
    "members",
    "meta",
    "operation_base",
    "operations",
    "organization",
    "organization_base",
    "organization_webhook_base",
    "organization_webhook_events",
    "organization_webhooks",
    "permissions",
    "permissionset",
    "permissionset_base",
    "providers_generated",
    "role_base",
    "roles",
    "status",
    "sub_orgs",
    "token_base",
    "tokens",
    "transforms",
    "usage",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AccountsActions,
    AccountsPermissions,
    AlarmPoliciesActions,
    AlarmPoliciesPermissions,
    AlarmsActions,
    AlarmsPermissions,
    ApiPermissionMap,
    AuditActions,
    AuditPermissions,
    AuthActions,
    AuthPermissions,
    BridgesActions,
    BridgesPermissions,
    CredentialsActions,
    CredentialsPermissions,
    GetPermissionSetResponse,
    IntegrationPointsActions,
    IntegrationPointsPermissions,
    IntegrationsActions,
    IntegrationsPermissions,
    ListPermissionSetsResponse,
    MembersActions,
    MembersPermissions,
    OperationsActions,
    OperationsPermissions,
    OrganizationActions,
    OrganizationPermissions,
    PermissionSet,
    PermissionSetActions,
    PermissionSetPermissions,
    ReadWriteActions,
    ReadWritePermissions,
    ResourceRestrictions,
    RolesActions,
    RolesPermissions,
    StatusActions,
    StatusPermissions,
    SubOrgsActions,
    SubOrgsPermissions,
    TokensActions,
    TokensPermissions,
    TransformsActions,
    TransformsPermissions,
    WebhooksActions,
    WebhooksPermissions,
)

__all__ = [
    "AccountsActions",
    "AccountsPermissions",
    "AlarmPoliciesActions",
    "AlarmPoliciesPermissions",
    "AlarmsActions",
    "AlarmsPermissions",
    "ApiPermissionMap",
    "AuditActions",
    "AuditPermissions",
    "AuthActions",
    "AuthPermissions",
    "BridgesActions",
    "BridgesPermissions",
    "CredentialsActions",
    "CredentialsPermissions",
    "GetPermissionSetResponse",
    "IntegrationPointsActions",
    "IntegrationPointsPermissions",
    "IntegrationsActions",
    "IntegrationsPermissions",
    "ListPermissionSetsResponse",
    "MembersActions",
    "MembersPermissions",
    "OperationsActions",
    "OperationsPermissions",
    "OrganizationActions",
    "OrganizationPermissions",
    "PermissionSet",
    "PermissionSetActions",
    "PermissionSetPermissions",
    "ReadWriteActions",
    "ReadWritePermissions",
    "ResourceRestrictions",
    "RolesActions",
    "RolesPermissions",
    "StatusActions",
    "StatusPermissions",
    "SubOrgsActions",
    "SubOrgsPermissions",
    "TokensActions",
    "TokensPermissions",
    "TransformsActions",
    "TransformsPermissions",
    "WebhooksActions",
    "WebhooksPermissions",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .credential_base import CredentialBase

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AwsCredential(CredentialBase):
    """
    AWS access key to authenticate with AWS. Access keys are long-term credentials for an IAM user and consist of an ID and secret. Follow [this guide to generate access and secret keys](https://docs.aws.amazon.com/general/latest/gr/aws-sec-cred-types.html#access-keys-and-secret-access-keys). You may optionally provide a session token if you are using temporary credentials.
    """

    access_key_id: str = pydantic.Field()
    """
    ID portion of the AWS access key pair.
    """

    secret_access_key: str = pydantic.Field()
    """
    Secret portion of the AWS access key pair.
    """

    session: typing.Optional[str] = pydantic.Field(default=None)
    """
    A temporary session token. Session tokens are optional and are only necessary if you are using temporary credentials.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...permissionset_base.types.permissions import Permissions
from ...role_base.types.resources import Resources

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CreateTokenRequest(pydantic.BaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique name token. If not provided, defaults to generated newly created refresh token id.
    """

    resources: Resources = pydantic.Field()
    """
    Limit access to supplied resources
    """

    permission_set: Permissions = pydantic.Field()
    """
    Limit access to supplied permissions
    """

    token_ttl: typing.Optional[str] = pydantic.Field(default=None)
    """
    Token time-to-live. If not provided, defaults to the TTL of the token used to call this API. Use the format "1h", "1m", "1s" for hours, minutes, and seconds respectively.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.timestamp import Timestamp
from ...objects.types.actor import Actor
from ...objects.types.agent import Agent
from ...objects.types.api import Api
from ...objects.types.cloud import Cloud
from ...objects.types.device import Device
from ...objects.types.enrichment import Enrichment
from ...objects.types.metadata import Metadata
from ...objects.types.object import Object
from ...objects.types.observable import Observable
from ...objects.types.osint import Osint
from ...objects.types.policy import Policy
from ...objects.types.scan import Scan
from .activity_id import ActivityId
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .severity_id import SeverityId
from .status_id import StatusId
from .type_uid import TypeUid

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ScanActivity(pydantic.BaseModel):
    """
    Scan events report the start, completion, and results of a scan job. The scan event includes the number of items that were scanned and the number of detections that were resolved.
    """

    activity_id: ActivityId = pydantic.Field()
    """
    The normalized identifier of the activity that triggered the event.
    """

    activity_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event activity name, as defined by the activity_id.
    """

    actor: typing.Optional[Actor] = pydantic.Field(default=None)
    """
    The actor object describes details about the user/role/process that was the source of the activity.
    """

    agent_list: typing.Optional[typing.List[Agent]] = pydantic.Field(default=None)
    """
    The agents that were used to scan the devices.
    """

    api: typing.Optional[Api] = pydantic.Field(default=None)
    """
    Describes details about a typical API (Application Programming Interface) call.
    """

    category_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event category name, as defined by category_uid value: <code>Application Activity</code>.
    """

    category_uid: CategoryUid = pydantic.Field()
    """
    The category unique identifier of the event.
    """

    class_uid: ClassUid = pydantic.Field()
    """
    The unique identifier of a class. A class describes the attributes available in an event.
    """

    cloud: typing.Optional[Cloud] = pydantic.Field(default=None)
    """
    Describes details about the Cloud environment where the event was originally created or logged.
    """

    command_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The command identifier that is associated with this scan event. This ID uniquely identifies the proactive scan command, e.g., if remotely initiated.
    """

    count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of times that events in the same logical group occurred during the event <strong>Start Time</strong> to <strong>End Time</strong> period.
    """

    custom_fields: typing.Optional[Object] = pydantic.Field(default=None)
    """
    A list of custom fields
    """

    device: typing.Optional[Device] = pydantic.Field(default=None)
    """
    An addressable device, computer system or host.
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The duration of the scan
    """

    end_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The end time of the scan job.
    """

    end_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The end time of the scan job.
    """

    enrichments: typing.Optional[typing.List[Enrichment]] = pydantic.Field(default=None)
    """
    The additional information from an external data source, which is associated with the event or a finding. For example add location information for the IP address in the DNS answers:</p><code>[{"name": "answers.ip", "value": "92.24.47.250", "type": "location", "data": {"city": "Socotra", "continent": "Asia", "coordinates": [-25.4153, 17.0743], "country": "YE", "desc": "Yemen"}}]</code>
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    metadata: Metadata = pydantic.Field()
    """
    The metadata associated with the event or a finding.
    """

    num_detections: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of detections.
    """

    num_files: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of files scanned.
    """

    num_folders: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of folders scanned.
    """

    num_hosts: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of hosts that were scanned.
    """

    num_network_items: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of network items scanned.
    """

    num_processes: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of processes scanned.
    """

    num_registry_items: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of registry items scanned.
    """

    num_resolutions: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of items that were resolved.
    """

    num_skipped_items: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of skipped items.
    """

    num_trusted_items: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of trusted items.
    """

    observables: typing.Optional[typing.List[Observable]] = pydantic.Field(default=None)
    """
    The observables associated with the event or a finding.
    """

    osint: typing.Optional[typing.List[Osint]] = pydantic.Field(default=None)
    """
    The OSINT (Open Source Intelligence) object contains details related to an indicator such as the indicator itself, related indicators, geolocation, registrar information, subdomains, analyst commentary, and other contextual information. This information can be used to further enrich a detection or finding by providing decisioning support to other analysts and engineers.
    """

    policy: typing.Optional[Policy] = pydantic.Field(default=None)
    """
    The policy associated with this Scan event; required if the scan was initiated by a policy.
    """

    raw_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    The raw event/finding data as received from the source.
    """

    scan: Scan = pydantic.Field()
    """
    The Scan object describes characteristics of the scan job.
    """

    schedule_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the schedule associated with a scan job.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding severity, normalized to the caption of the severity_id value. In the case of 'Other', it is defined by the source.
    """

    severity_id: SeverityId = pydantic.Field()
    """
    <p>The normalized identifier of the event/finding severity.</p>The normalized severity is a measurement the effort and expense required to manage and resolve an event or incident. Smaller numerical values represent lower impact events, and larger numerical values represent higher impact events.
    """

    start_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The start time of the scan job.
    """

    start_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The start time of the scan job.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status, normalized to the caption of the status_id value. In the case of 'Other', it is defined by the event source.
    """

    status_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status code, as reported by the event source.<br /><br />For example, in a Windows Failed Authentication event, this would be the value of 'Failure Code', e.g. 0x18.
    """

    status_detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status detail contains additional information about the event/finding outcome.
    """

    status_id: typing.Optional[StatusId] = pydantic.Field(default=None)
    """
    The normalized identifier of the event status.
    """

    time: Timestamp = pydantic.Field()
    """
    The normalized event occurrence time or the finding creation time.
    """

    time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The normalized event occurrence time or the finding creation time.
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of minutes that the reported event <code>time</code> is ahead or behind UTC, in the range -1,080 to +1,080.
    """

    total: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of items that were scanned; zero if no items were scanned.
    """

    type_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding type name, as defined by the type_uid.
    """

    type_uid: TypeUid = pydantic.Field()
    """
    The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: <code>class_uid \* 100 + activity_id</code>.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the event schema. The names and values of those attributes are specific to the event source.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import Base, ErrorBody, ErrorParam, Id, PatchOp, PatchOperation
from .errors import (
    BadGatewayError,
    BadRequestError,
    ConflictError,
    ForbiddenError,
    GatewayTimeoutError,
    InternalServerError,
    MethodNotAllowedError,
    NotFoundError,
    NotImplementedError,
    ServiceUnavailableError,
    TooManyRequestsError,
    UnauthorizedError,
    UnsupportedMediaTypeError,
)

__all__ = [
    "BadGatewayError",
    "BadRequestError",
    "Base",
    "ConflictError",
    "ErrorBody",
    "ErrorParam",
    "ForbiddenError",
    "GatewayTimeoutError",
    "Id",
    "InternalServerError",
    "MethodNotAllowedError",
    "NotFoundError",
    "NotImplementedError",
    "PatchOp",
    "PatchOperation",
    "ServiceUnavailableError",
    "TooManyRequestsError",
    "UnauthorizedError",
    "UnsupportedMediaTypeError",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.email_address import EmailAddress
from ...base.types.ip_address import IpAddress

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Email(pydantic.BaseModel):
    """
    The Email object describes the email metadata such as sender, recipients, and direction. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:Email/'>d3f:Email</a>.
    """

    cc: typing.Optional[typing.List[EmailAddress]] = pydantic.Field(default=None)
    """
    The email header Cc values, as defined by RFC 5322.
    """

    delivered_to: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The <strong>Delivered-To</strong> email header field.
    """

    from_: EmailAddress = pydantic.Field(alias="from")
    """
    The email header From values, as defined by RFC 5322.
    """

    message_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email header Message-Id value, as defined by RFC 5322.
    """

    raw_header: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email authentication header.
    """

    reply_to: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The email header Reply-To values, as defined by RFC 5322.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size in bytes of the email, including attachments.
    """

    smtp_from: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The value of the SMTP MAIL FROM command.
    """

    smtp_to: typing.Optional[typing.List[EmailAddress]] = pydantic.Field(default=None)
    """
    The value of the SMTP envelope RCPT TO command.
    """

    subject: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email header Subject value, as defined by RFC 5322.
    """

    to: typing.List[EmailAddress] = pydantic.Field()
    """
    The email header To values, as defined by RFC 5322.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email unique identifier.
    """

    x_originating_ip: typing.Optional[typing.List[IpAddress]] = pydantic.Field(default=None)
    """
    The X-Originating-IP header identifying the emails originating IP address(es).
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.email_address import EmailAddress
from ...base.types.user_name import UserName
from .account import Account
from .group import Group
from .ldap_person import LdapPerson
from .organization import Organization
from .user_mfa_status_id import UserMfaStatusId
from .user_type_id import UserTypeId
from .user_user_status_id import UserUserStatusId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class User(pydantic.BaseModel):
    """
    The User object describes the characteristics of a user/person or a security principal. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:UserAccount/'>d3f:UserAccount</a>.
    """

    account: typing.Optional[Account] = pydantic.Field(default=None)
    """
    The user's account or the account associated with the user.
    """

    credential_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the user's credential. For example, AWS Access Key ID.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The domain where the user is defined. For example: the LDAP or Active Directory domain.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The user's primary email address.
    """

    full_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full name of the person, as per the LDAP Common Name attribute (cn).
    """

    groups: typing.Optional[typing.List[Group]] = pydantic.Field(default=None)
    """
    The administrative groups to which the user belongs.
    """

    ldap_person: typing.Optional[LdapPerson] = pydantic.Field(default=None)
    """
    The additional LDAP attributes that describe a person.
    """

    mfa_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The multi-factor authentication status, normalized to the caption of the mfa_status_id value. In the case of 'Other', it is defined by the data source.
    """

    mfa_status_id: typing.Optional[UserMfaStatusId] = pydantic.Field(default=None)
    """
    The normalized identifier of the user's multi-factor authentication status.
    """

    name: typing.Optional[UserName] = pydantic.Field(default=None)
    """
    The username. For example, <code>janedoe1</code>.
    """

    org: typing.Optional[Organization] = pydantic.Field(default=None)
    """
    Organization and org unit related to the user.
    """

    privileges: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The user's privileges.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the user. For example, System, AWS IAM User, etc.
    """

    type_id: typing.Optional[UserTypeId] = pydantic.Field(default=None)
    """
    The account type identifier.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique user identifier. For example, the Windows user SID, ActiveDirectory DN or AWS user ARN.
    """

    uid_alt: typing.Optional[str] = pydantic.Field(default=None)
    """
    The alternate user identifier. For example, the Active Directory user GUID or AWS user Principal ID.
    """

    user_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The user status, normalized to the caption of the user_status_id value. In the case of 'Other', it is defined by the data source.
    """

    user_status_id: typing.Optional[UserUserStatusId] = pydantic.Field(default=None)
    """
    The normalized identifier of the user's status.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .create_transform_request import CreateTransformRequest
from .create_transform_response import CreateTransformResponse
from .get_transform_response import GetTransformResponse
from .list_transforms_response import ListTransformsResponse
from .patch_transform_response import PatchTransformResponse
from .transform import Transform
from .transform_id import TransformId
from .update_transform_request import UpdateTransformRequest
from .update_transform_response import UpdateTransformResponse

__all__ = [
    "CreateTransformRequest",
    "CreateTransformResponse",
    "GetTransformResponse",
    "ListTransformsResponse",
    "PatchTransformResponse",
    "Transform",
    "TransformId",
    "UpdateTransformRequest",
    "UpdateTransformResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.url_string import UrlString

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class WebResource(pydantic.BaseModel):
    """
    The Web Resource object describes characteristics of a web resource that was affected by the activity/event.
    """

    data: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    Details of the web resource, e.g, <code>file</code> details, <code>search</code> results or application-defined resource.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the web resource.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of labels/tags associated to a resource.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the web resource.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The web resource type as defined by the event source.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the web resource.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The URL pointing towards the source of the web resource.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .agent import Agent
from .group import Group
from .user import User

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ResourceDetails(pydantic.BaseModel):
    """
    The Resource Details object describes details about resources that were affected by the activity/event.
    """

    agent_list: typing.Optional[typing.List[Agent]] = pydantic.Field(default=None)
    """
    A list of <code>agent</code> objects associated with a device, endpoint, or resource.
    """

    cloud_partition: typing.Optional[str] = pydantic.Field(default=None)
    """
    The canonical cloud partition name to which the region is assigned (e.g. AWS Partitions: aws, aws-cn, aws-us-gov).
    """

    criticality: typing.Optional[str] = pydantic.Field(default=None)
    """
    The criticality of the resource as defined by the event source.
    """

    data: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    Additional data describing the resource.
    """

    group: typing.Optional[Group] = pydantic.Field(default=None)
    """
    The name of the related resource group.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of labels/tags associated to a resource.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the resource.
    """

    namespace: typing.Optional[str] = pydantic.Field(default=None)
    """
    The namespace is useful when similar entities exist that you need to keep separate.
    """

    owner: typing.Optional[User] = pydantic.Field(default=None)
    """
    The identity of the service or user account that owns the resource.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cloud region of the resource.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The resource type as defined by the event source.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the resource.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the resource. For example <code>1.2.3</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.processactivity.resources.classes.types.process_activity import (
    ProcessActivity as resources_ocsf_resources_v_1_3_0_resources_processactivity_resources_classes_types_process_activity_ProcessActivity,
)

ProcessActivity = (
    resources_ocsf_resources_v_1_3_0_resources_processactivity_resources_classes_types_process_activity_ProcessActivity
)



# This file was auto-generated by Fern from our API Definition.

from .types import ActivityId, CategoryUid, ClassUid, ConfidenceId, SeverityId, StatusId, TypeUid, VulnerabilityFinding

__all__ = [
    "ActivityId",
    "CategoryUid",
    "ClassUid",
    "ConfidenceId",
    "SeverityId",
    "StatusId",
    "TypeUid",
    "VulnerabilityFinding",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .scan_type_id import ScanTypeId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Scan(pydantic.BaseModel):
    """
    The Scan object describes characteristics of a proactive scan.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The administrator-supplied or application-generated name of the scan. For example: "Home office weekly user database scan", "Scan folders for viruses", "Full system virus scan"
    """

    targets: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Hosts or IP addresses targeted by the scan.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of scan.
    """

    type_id: ScanTypeId = pydantic.Field()
    """
    The type id of the scan.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The application-defined unique identifier assigned to an instance of a scan.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .account import Account
from .account_type_id import AccountTypeId
from .actor import Actor
from .api import Api
from .attack import Attack
from .authorization import Authorization
from .certificate import Certificate
from .cloud import Cloud
from .container import Container
from .cve import Cve
from .cvss import Cvss
from .cvss_depth import CvssDepth
from .cwe import Cwe
from .device import Device
from .device_hw_info import DeviceHwInfo
from .device_network_status_id import DeviceNetworkStatusId
from .device_risk_level_id import DeviceRiskLevelId
from .device_type_id import DeviceTypeId
from .digital_signature import DigitalSignature
from .digital_signature_algorithm_id import DigitalSignatureAlgorithmId
from .display import Display
from .enrichment import Enrichment
from .epss import Epss
from .extension import Extension
from .feature import Feature
from .file import File
from .file_confidentiality_id import FileConfidentialityId
from .file_type_id import FileTypeId
from .fingerprint import Fingerprint
from .fingerprint_algorithm_id import FingerprintAlgorithmId
from .firewall_rule import FirewallRule
from .group import Group
from .idp import Idp
from .image import Image
from .keyboard_info import KeyboardInfo
from .ldap_person import LdapPerson
from .location import Location
from .logger import Logger
from .malware import Malware
from .malware_classification_ids import MalwareClassificationIds
from .metadata import Metadata
from .metric import Metric
from .module import Module
from .module_load_type_id import ModuleLoadTypeId
from .network_interface import NetworkInterface
from .network_interface_type_id import NetworkInterfaceTypeId
from .object import Object
from .observable import Observable
from .observable_type_id import ObservableTypeId
from .organization import Organization
from .os import Os
from .os_type_id import OsTypeId
from .policy import Policy
from .process import Process
from .process_integrity_id import ProcessIntegrityId
from .product import Product
from .reputation import Reputation
from .reputation_score_id import ReputationScoreId
from .request import Request
from .response import Response
from .service import Service
from .session import Session
from .sub_technique import SubTechnique
from .tactic import Tactic
from .technique import Technique
from .user import User
from .user_mfa_status_id import UserMfaStatusId
from .user_type_id import UserTypeId
from .user_user_status_id import UserUserStatusId

__all__ = [
    "Account",
    "AccountTypeId",
    "Actor",
    "Api",
    "Attack",
    "Authorization",
    "Certificate",
    "Cloud",
    "Container",
    "Cve",
    "Cvss",
    "CvssDepth",
    "Cwe",
    "Device",
    "DeviceHwInfo",
    "DeviceNetworkStatusId",
    "DeviceRiskLevelId",
    "DeviceTypeId",
    "DigitalSignature",
    "DigitalSignatureAlgorithmId",
    "Display",
    "Enrichment",
    "Epss",
    "Extension",
    "Feature",
    "File",
    "FileConfidentialityId",
    "FileTypeId",
    "Fingerprint",
    "FingerprintAlgorithmId",
    "FirewallRule",
    "Group",
    "Idp",
    "Image",
    "KeyboardInfo",
    "LdapPerson",
    "Location",
    "Logger",
    "Malware",
    "MalwareClassificationIds",
    "Metadata",
    "Metric",
    "Module",
    "ModuleLoadTypeId",
    "NetworkInterface",
    "NetworkInterfaceTypeId",
    "Object",
    "Observable",
    "ObservableTypeId",
    "Organization",
    "Os",
    "OsTypeId",
    "Policy",
    "Process",
    "ProcessIntegrityId",
    "Product",
    "Reputation",
    "ReputationScoreId",
    "Request",
    "Response",
    "Service",
    "Session",
    "SubTechnique",
    "Tactic",
    "Technique",
    "User",
    "UserMfaStatusId",
    "UserTypeId",
    "UserUserStatusId",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .sub_technique import SubTechnique
from .tactic import Tactic
from .technique import Technique

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Attack(pydantic.BaseModel):
    """
    The <a target='_blank' href='https://attack.mitre.org'>MITRE ATT&CK®</a> object describes the tactic, technique & sub-technique associated to an attack as defined in <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK Matrix<sup>TM</sup></a>.
    """

    sub_technique: typing.Optional[SubTechnique] = pydantic.Field(default=None)
    """
    The Sub Technique object describes the sub technique ID and/or name associated to an attack, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK Matrix<sup>TM</sup></a>.
    """

    tactic: typing.Optional[Tactic] = pydantic.Field(default=None)
    """
    The Tactic object describes the tactic ID and/or name that is associated to an attack, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK Matrix<sup>TM</sup></a>.
    """

    tactics: typing.Optional[typing.List[Tactic]] = pydantic.Field(default=None)
    """
    The Tactic object describes the tactic ID and/or tactic name that are associated with the attack technique, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK Matrix<sup>TM</sup></a>.
    """

    technique: typing.Optional[Technique] = pydantic.Field(default=None)
    """
    The Technique object describes the technique ID and/or name associated to an attack, as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK Matrix<sup>TM</sup></a>.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK Matrix<sup>TM</sup></a> version.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class FirewallRule(pydantic.BaseModel):
    """
    The Firewall Rule object represents a specific rule within a firewall policy or event. It contains information about a rule's configuration, properties, and associated actions that define how network traffic is handled by the firewall.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule category.
    """

    condition: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule trigger condition for the rule. For example: SQL_INJECTION.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the rule that generated the event.
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The rule response time duration, usually used for challenge completion time.
    """

    match_details: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The data in a request that rule matched. For example: '["10","and","1"]'.
    """

    match_location: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location of the matched data in the source which resulted in the triggered firewall rule. For example: HEADER.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the rule that generated the event.
    """

    rate_limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    The rate limit for a rate-based rule.
    """

    sensitivity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The sensitivity of the firewall rule in the matched event. For example: HIGH.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule type.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the rule that generated the event.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The rule version. For example: <code>1.1</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .container import Container

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Response(pydantic.BaseModel):
    """
    The Response Elements object describes characteristics of an API response.
    """

    code: typing.Optional[int] = pydantic.Field(default=None)
    """
    The numeric response sent to a request.
    """

    containers: typing.Optional[typing.List[Container]] = pydantic.Field(default=None)
    """
    When working with containerized applications, the set of containers which write to the standard the output of a particular logging driver. For example, this may be the set of containers involved in handling api requests and responses for a containerized application.
    """

    data: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    The additional data that is associated with the api response.
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error Code
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error Message
    """

    flags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of communication flags, normalized to the captions of the flag_ids values. In the case of 'Other', they are defined by the event source.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .create_member_request import CreateMemberRequest
from .create_member_response import CreateMemberResponse
from .create_member_response_result import CreateMemberResponseResult
from .member import Member
from .member_id import MemberId
from .member_options import MemberOptions
from .options import Options
from .state import State

__all__ = [
    "CreateMemberRequest",
    "CreateMemberResponse",
    "CreateMemberResponseResult",
    "Member",
    "MemberId",
    "MemberOptions",
    "Options",
    "State",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .file import File
from .module_load_type_id import ModuleLoadTypeId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Module(pydantic.BaseModel):
    """
    The Module object describes the load attributes of a module.
    """

    base_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    The memory address where the module was loaded.
    """

    file: typing.Optional[File] = pydantic.Field(default=None)
    """
    The module file object.
    """

    function_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The entry-point function of the module. The system calls the entry-point function whenever a process or thread loads or unloads the module.
    """

    load_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The load type, normalized to the caption of the load_type_id value. In the case of 'Other', it is defined by the event source. It describes how the module was loaded in memory.
    """

    load_type_id: ModuleLoadTypeId = pydantic.Field()
    """
    The normalized identifier of the load type. It identifies how the module was loaded in memory.
    """

    start_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    The start address of the execution.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The module type.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...credentials.types.bridge_token_credential import BridgeTokenCredential
from ...credentials.types.bridge_token_credential_id import BridgeTokenCredentialId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SplunkBridgeHecToken_BridgeToken(BridgeTokenCredential):
    type: typing.Literal["bridge_token"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class SplunkBridgeHecToken_BridgeTokenId(pydantic.BaseModel):
    type: typing.Literal["bridge_token_id"]
    value: BridgeTokenCredentialId

    class Config:
        frozen = True
        smart_union = True


SplunkBridgeHecToken = typing.Union[SplunkBridgeHecToken_BridgeToken, SplunkBridgeHecToken_BridgeTokenId]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...account_base.types.account_id import AccountId
from ...accounts.types.account import Account
from ...capabilities_base.types.category_id import CategoryId
from ...common.types.base import Base
from ...integration_base.types.integration_id import IntegrationId
from ...integration_points.types.integration_point import IntegrationPoint
from ...integration_points.types.integration_point_id import IntegrationPointId
from ...providers_generated.types.provider_config import ProviderConfig
from ...token_base.types.token_id import TokenId
from .bridge_selector import BridgeSelector
from .webhook_config import WebhookConfig

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Integration(Base):
    """
    Connects an Account to an external service
    """

    id: IntegrationId
    fullname: str = pydantic.Field()
    """
    Human friendly display name for this integration.
    """

    refresh_token_id: TokenId = pydantic.Field()
    """
    Integration refresh token id
    """

    account_id: AccountId = pydantic.Field()
    """
    Account associated with this integration. Use the expand=accounts parameter with the List and ListAccount APIs to expand the Account to the full object
    """

    account: typing.Optional[Account] = pydantic.Field(default=None)
    """
    When using the expand option on the List or ListAccount APIs, the full account object is included in the response
    """

    category: CategoryId = pydantic.Field()
    """
    Id of the Connector Category for this Integration.
    """

    provider_config: ProviderConfig = pydantic.Field()
    """
    Provider configuration for this Integration.
    """

    provider_fullname: str = pydantic.Field()
    """
    Human friendly display name for the provider.
    """

    provider_type: str = pydantic.Field()
    """
    Type of the provider for this Integration.
    """

    integration_point_id: typing.Optional[IntegrationPointId] = pydantic.Field(default=None)
    """
    Integration Point associated with this integration. Use the expand=integration_points parameter with the List and ListAccount APIs to expand the Integration Point to the full object
    """

    integration_point: typing.Optional[IntegrationPoint] = pydantic.Field(default=None)
    """
    When using the expand option on the List or ListAccount APIs, the full integration_point object is included in the response
    """

    bridge_selector: typing.Optional[BridgeSelector] = pydantic.Field(default=None)
    """
    Use a Bridge to connect to the provider.
    """

    webhook_config: typing.Optional[WebhookConfig] = pydantic.Field(default=None)
    """
    Webhook configuration for this integration. Some providers support webhooks, and will allow end users providers to send events to a server for new or updated data.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .sumo_logic_collection_url import SumoLogicCollectionUrl
from .sumo_logic_credential import SumoLogicCredential

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SiemSumoLogic(pydantic.BaseModel):
    """
    Configuration for Sumo Logic Cloud SIEM.
    """

    auto_parse_logs: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Automatically parse logs as JSON when running log queries against Sumo Logic.
    """

    collection_url: typing.Optional[SumoLogicCollectionUrl] = pydantic.Field(default=None)
    """
    Required if you need to send Sumo Logic events from the Synqly API.
    """

    credential: SumoLogicCredential
    siem_logs_only: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Only query for logs that have been processed into the Sumo Logic Cloud SIEM app.
    """

    url: str = pydantic.Field()
    """
    Your Sumo Logic API endpoint. See https://help.sumologic.com/docs/api/getting-started/#sumo-logic-endpoints-by-deployment-and-firewall-security for help determining which base URL to use.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from ...common.types.common_properties import CommonProperties
from ...common.types.kill_chain_phase import KillChainPhase
from ...open_vocab.types.indicator_type_ov import IndicatorTypeOv
from ...open_vocab.types.pattern_type_ov import PatternTypeOv

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Indicator(CommonProperties):
    type: typing.Literal["indicator"]
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    A name used to identify the Indicator.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description that provides more details and context about the Indicator, potentially including its purpose and its key characteristics.
    """

    indicator_types: typing.Optional[typing.List[IndicatorTypeOv]] = pydantic.Field(default=None)
    """
    A set of categorizations for this indicator.
    """

    pattern: typing.Optional[str] = pydantic.Field(default=None)
    """
    The detection pattern for this Indicator MAY be expressed as a STIX Pattern as specified in section 9 or another appropriate language such as SNORT, YARA, etc.
    """

    pattern_type: typing.Optional[PatternTypeOv] = pydantic.Field(default=None)
    """
    The type of pattern expression used in the pattern property.
    """

    pattern_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the pattern expression used in the pattern property.
    """

    valid_from: dt.datetime = pydantic.Field()
    """
    The time from which this Indicator is considered a valid indicator of the behaviors it is related or represents.
    """

    valid_until: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time at which this Indicator is no longer considered valid.
    """

    kill_chain_phases: typing.Optional[typing.List[KillChainPhase]] = pydantic.Field(default=None)
    """
    The kill chain phase to which this Indicator corresponds.
    """

    raw_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    The data as received from the source.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TokensActions(str, enum.Enum):
    LIST = "list"
    CREATE = "create"
    CREATE_INTEGRATION = "create_integration"
    CREATE_SYNQLY_INTEGRATIONS = "create_synqly_integrations"
    DELETE = "delete"
    GET = "get"
    RESET = "reset"
    REFRESH = "refresh"
    REMOVE_SECONDARY = "remove_secondary"
    ALL = "*"

    def visit(
        self,
        list_: typing.Callable[[], T_Result],
        create: typing.Callable[[], T_Result],
        create_integration: typing.Callable[[], T_Result],
        create_synqly_integrations: typing.Callable[[], T_Result],
        delete: typing.Callable[[], T_Result],
        get: typing.Callable[[], T_Result],
        reset: typing.Callable[[], T_Result],
        refresh: typing.Callable[[], T_Result],
        remove_secondary: typing.Callable[[], T_Result],
        all_: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is TokensActions.LIST:
            return list_()
        if self is TokensActions.CREATE:
            return create()
        if self is TokensActions.CREATE_INTEGRATION:
            return create_integration()
        if self is TokensActions.CREATE_SYNQLY_INTEGRATIONS:
            return create_synqly_integrations()
        if self is TokensActions.DELETE:
            return delete()
        if self is TokensActions.GET:
            return get()
        if self is TokensActions.RESET:
            return reset()
        if self is TokensActions.REFRESH:
            return refresh()
        if self is TokensActions.REMOVE_SECONDARY:
            return remove_secondary()
        if self is TokensActions.ALL:
            return all_()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .nucleus_credential import NucleusCredential

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class VulnerabilitiesNucleus(pydantic.BaseModel):
    """
    Configuration for Nucleus as a Vulnerabilities Provider
    """

    credential: NucleusCredential
    project_id: str = pydantic.Field()
    """
    The **project_id** is a string representing a numeric ID (e.g., "12345") and can be found in the Nucleus UI by selecting **Global Dashboard** from the left-hand menu, then choosing **All Projects** at the top, where the **Projects** widget lists all projects with their IDs.
    """

    url: str = pydantic.Field()
    """
    URL for the Nucleus API. This should be the base URL for the API, without any path components and must be HTTPS, e.g. "https://{sandbox}.nucleussec.com" .
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.timestamp import Timestamp
from ...base.types.url_string import UrlString
from ...objects.types.api import Api
from ...objects.types.attack import Attack
from ...objects.types.cloud import Cloud
from ...objects.types.enrichment import Enrichment
from ...objects.types.finding_info import FindingInfo
from ...objects.types.group import Group
from ...objects.types.metadata import Metadata
from ...objects.types.object import Object
from ...objects.types.observable import Observable
from ...objects.types.user import User
from .activity_id import ActivityId
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .confidence_id import ConfidenceId
from .impact_id import ImpactId
from .priority_id import PriorityId
from .severity_id import SeverityId
from .status_id import StatusId
from .type_uid import TypeUid
from .verdict_id import VerdictId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class IncidentFinding(pydantic.BaseModel):
    """
    An Incident Finding reports the creation, update, or closure of security incidents as a result of detections and/or analytics.
    """

    activity_id: ActivityId = pydantic.Field()
    """
    The normalized identifier of the Incident activity.
    """

    activity_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Incident activity name, as defined by the <code>activity_id</code>.
    """

    api: typing.Optional[Api] = pydantic.Field(default=None)
    """
    Describes details about a typical API (Application Programming Interface) call.
    """

    assignee: typing.Optional[User] = pydantic.Field(default=None)
    """
    The details of the user assigned to an Incident.
    """

    assignee_group: typing.Optional[Group] = pydantic.Field(default=None)
    """
    The details of the group assigned to an Incident.
    """

    attacks: typing.Optional[typing.List[Attack]] = pydantic.Field(default=None)
    """
    An array of <a target='_blank' href='https://attack.mitre.org'>MITRE ATT&CK®</a> objects describing the tactics, techniques & sub-techniques associated to the Incident.
    """

    category_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event category name, as defined by category_uid value: <code>Findings</code>.
    """

    category_uid: CategoryUid = pydantic.Field()
    """
    The category unique identifier of the event.
    """

    class_uid: ClassUid = pydantic.Field()
    """
    The unique identifier of a class. A Class describes the attributes available in an event.
    """

    cloud: typing.Optional[Cloud] = pydantic.Field(default=None)
    """
    Describes details about the Cloud environment where the event was originally created or logged.
    """

    comment: typing.Optional[str] = pydantic.Field(default=None)
    """
    Additional user supplied details for updating or closing the incident.
    """

    confidence: typing.Optional[str] = pydantic.Field(default=None)
    """
    The confidence, normalized to the caption of the confidence_id value. In the case of 'Other', it is defined by the event source.
    """

    confidence_id: typing.Optional[ConfidenceId] = pydantic.Field(default=None)
    """
    The normalized confidence refers to the accuracy of the rule that created the finding. A rule with a low confidence means that the finding scope is wide and may create finding reports that may not be malicious in nature.
    """

    confidence_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The confidence score as reported by the event source.
    """

    count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of times that events in the same logical group occurred during the event <strong>Start Time</strong> to <strong>End Time</strong> period.
    """

    custom_fields: typing.Optional[Object] = pydantic.Field(default=None)
    """
    A list of custom fields
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The short description of the Incident.
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The event duration or aggregate time, the amount of time the event covers from <code>start_time</code> to <code>end_time</code> in milliseconds.
    """

    end_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time of the most recent event included in the incident.
    """

    end_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time of the most recent event included in the incident.
    """

    enrichments: typing.Optional[typing.List[Enrichment]] = pydantic.Field(default=None)
    """
    The additional information from an external data source, which is associated with the event or a finding. For example add location information for the IP address in the DNS answers:</p><code>[{"name": "answers.ip", "value": "92.24.47.250", "type": "location", "data": {"city": "Socotra", "continent": "Asia", "coordinates": [-25.4153, 17.0743], "country": "YE", "desc": "Yemen"}}]</code>
    """

    finding_info_list: typing.List[FindingInfo] = pydantic.Field()
    """
    A list of <code>finding_info</code> objects associated to an incident.
    """

    impact: typing.Optional[str] = pydantic.Field(default=None)
    """
    The impact , normalized to the caption of the impact_id value. In the case of 'Other', it is defined by the event source.
    """

    impact_id: typing.Optional[ImpactId] = pydantic.Field(default=None)
    """
    The normalized impact of the finding.
    """

    impact_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The impact of the finding, valid range 0-100.
    """

    is_suspected_breach: typing.Optional[bool] = pydantic.Field(default=None)
    """
    A determination based on analytics as to whether a potential breach was found.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    metadata: Metadata = pydantic.Field()
    """
    The metadata associated with the event or a finding.
    """

    observables: typing.Optional[typing.List[Observable]] = pydantic.Field(default=None)
    """
    The observables associated with the event or a finding.
    """

    priority: typing.Optional[int] = pydantic.Field(default=None)
    """
    The priority, normalized to the caption of the priority_id value. In the case of 'Other', it is defined by the event source.
    """

    priority_id: typing.Optional[PriorityId] = pydantic.Field(default=None)
    """
    The normalized priority. Priority identifies the relative importance of the finding. It is a measurement of urgency.
    """

    raw_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    The raw event/finding data as received from the source.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding severity, normalized to the caption of the severity_id value. In the case of 'Other', it is defined by the source.
    """

    severity_id: SeverityId = pydantic.Field()
    """
    <p>The normalized identifier of the event/finding severity.</p>The normalized severity is a measurement the effort and expense required to manage and resolve an event or incident. Smaller numerical values represent lower impact events, and larger numerical values represent higher impact events.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    A Url link used to access the original incident.
    """

    start_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time of the least recent event included in the incident.
    """

    start_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time of the least recent event included in the incident.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The normalized status of the Incident normalized to the caption of the status_id value. In the case of 'Other', it is defined by the source.
    """

    status_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status code, as reported by the event source.<br /><br />For example, in a Windows Failed Authentication event, this would be the value of 'Failure Code', e.g. 0x18.
    """

    status_detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status details contains additional information about the event/finding outcome.
    """

    status_id: StatusId = pydantic.Field()
    """
    The normalized status identifier of the Incident.
    """

    time: Timestamp = pydantic.Field()
    """
    The normalized event occurrence time or the finding creation time.
    """

    time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The normalized event occurrence time or the finding creation time.
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of minutes that the reported event <code>time</code> is ahead or behind UTC, in the range -1,080 to +1,080.
    """

    type_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding type name, as defined by the type_uid.
    """

    type_uid: TypeUid = pydantic.Field()
    """
    The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: <code>class_uid \* 100 + activity_id</code>.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the event schema. The names and values of those attributes are specific to the event source.
    """

    verdict: typing.Optional[str] = pydantic.Field(default=None)
    """
    The verdict assigned to an Incident finding.
    """

    verdict_id: typing.Optional[VerdictId] = pydantic.Field(default=None)
    """
    The normalized verdict of an Incident.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .options import Options

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class MemberOptions(pydantic.BaseModel):
    ttl: str = pydantic.Field()
    """
    Optional member time-to-live duration. After a member expires, system requires a change password to re-enable member. Minimum 1 day, Maximum 1 year, Default 180 days.
    """

    options: typing.List[Options] = pydantic.Field()
    """
    Options: "expired" will force change password on first logon.
    """

    token_ttl: str = pydantic.Field()
    """
    Optional token time-to-live duration. Tokens are created for this member with this duration as their TTL. Minimum 10 miniutes, Maximum 1 week, Defaults 1 hour.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...capabilities_base.types.category_id import CategoryId
from .integration_environments import IntegrationEnvironments

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CreateIntegrationPointRequest(pydantic.BaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique short name for this Integration Point (lowercase [a-z0-9_-], can be used in URLs). Also used for case insensitive duplicate name detection and default sort order. Defaults to IntegrationPointId if both name and fullname are not specified.
    """

    fullname: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of integration point, will be shown to end-users in the Connect UI. Defaults to the same value as the 'name' field if not specified.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional description of the Integration Point. Will not be displayed to end-users of Connect UI.
    """

    connector: CategoryId = pydantic.Field()
    """
    Connector to use for the Integration Point.
    """

    environments: IntegrationEnvironments = pydantic.Field()
    """
    Selects providers to use for account environments.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateIntegrationPointRequest,
    CreateIntegrationPointResponse,
    GetIntegrationPointResponse,
    IntegrationEnvironments,
    IntegrationPoint,
    IntegrationPointId,
    ListIntegrationPointsResponse,
    PatchIntegrationPointResponse,
    UpdateIntegrationPointRequest,
    UpdateIntegrationPointResponse,
)

__all__ = [
    "CreateIntegrationPointRequest",
    "CreateIntegrationPointResponse",
    "GetIntegrationPointResponse",
    "IntegrationEnvironments",
    "IntegrationPoint",
    "IntegrationPointId",
    "ListIntegrationPointsResponse",
    "PatchIntegrationPointResponse",
    "UpdateIntegrationPointRequest",
    "UpdateIntegrationPointResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .application import Application
from .connection_state import ConnectionState
from .create_iocs_request import CreateIocsRequest
from .create_iocs_response import CreateIocsResponse
from .delete_iocs_response import DeleteIocsResponse
from .get_endpoint_response import GetEndpointResponse
from .network_quarantine_request import NetworkQuarantineRequest
from .network_quarantine_response import NetworkQuarantineResponse
from .posture_score import PostureScore
from .query_alerts_response import QueryAlertsResponse
from .query_applications_response import QueryApplicationsResponse
from .query_endpoints_response import QueryEndpointsResponse
from .query_iocs_response import QueryIocsResponse
from .query_posture_score_response import QueryPostureScoreResponse
from .query_threats_response import QueryThreatsResponse
from .threat_event import ThreatEvent

__all__ = [
    "Application",
    "ConnectionState",
    "CreateIocsRequest",
    "CreateIocsResponse",
    "DeleteIocsResponse",
    "GetEndpointResponse",
    "NetworkQuarantineRequest",
    "NetworkQuarantineResponse",
    "PostureScore",
    "QueryAlertsResponse",
    "QueryApplicationsResponse",
    "QueryEndpointsResponse",
    "QueryIocsResponse",
    "QueryPostureScoreResponse",
    "QueryThreatsResponse",
    "ThreatEvent",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ManagedType(str, enum.Enum):
    MANAGED = "Managed"
    UNMANAGED = "Unmanaged"

    def visit(self, managed: typing.Callable[[], T_Result], unmanaged: typing.Callable[[], T_Result]) -> T_Result:
        if self is ManagedType.MANAGED:
            return managed()
        if self is ManagedType.UNMANAGED:
            return unmanaged()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Location(pydantic.BaseModel):
    """
    The Geo Location object describes a geographical location, usually associated with an IP address. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:PhysicalLocation/'>d3f:PhysicalLocation</a>.
    """

    city: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the city.
    """

    continent: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the continent.
    """

    coordinates: typing.Optional[typing.List[float]] = pydantic.Field(default=None)
    """
    A two-element array, containing a longitude/latitude pair. The format conforms with <a target='_blank' href='https://geojson.org'>GeoJSON</a>. For example: <code>[-73.983, 40.719]</code>.
    """

    country: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ISO 3166-1 Alpha-2 country code. For the complete list of country codes see <a target='_blank' href='https://www.iso.org/obp/ui/#iso:pub:PUB500001:en' >ISO 3166-1 alpha-2 codes</a>.<p><b>Note:</b> The two letter country code should be capitalized. For example: <code>US</code> or <code>CA</code>.</p>
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the geographical location.
    """

    is_on_premises: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The indication of whether the location is on premises.
    """

    isp: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the Internet Service Provider (ISP).
    """

    postal_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The postal code of the location.
    """

    provider: typing.Optional[str] = pydantic.Field(default=None)
    """
    The provider of the geographical location data.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The alphanumeric code that identifies the principal subdivision (e.g. province or state) of the country. Region codes are defined at <a target='_blank' href='https://www.iso.org/iso-3166-country-codes.html'>ISO 3166-2</a> and have a limit of three characters. For example, see <a target='_blank' href='https://www.iso.org/obp/ui/#iso:code:3166:US'>the region codes for the US</a>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AuthCode,
    ChangePasswordResponse,
    ChangePasswordResponseResult,
    LogonRequest,
    LogonResponse,
    LogonResponseResult,
)

__all__ = [
    "AuthCode",
    "ChangePasswordResponse",
    "ChangePasswordResponseResult",
    "LogonRequest",
    "LogonResponse",
    "LogonResponseResult",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CredentialType(str, enum.Enum):
    AWS = "aws"
    TOKEN = "token"
    BASIC = "basic"
    SECRET = "secret"
    O_AUTH_CLIENT = "o_auth_client"
    BRIDGE = "bridge"

    def visit(
        self,
        aws: typing.Callable[[], T_Result],
        token: typing.Callable[[], T_Result],
        basic: typing.Callable[[], T_Result],
        secret: typing.Callable[[], T_Result],
        o_auth_client: typing.Callable[[], T_Result],
        bridge: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is CredentialType.AWS:
            return aws()
        if self is CredentialType.TOKEN:
            return token()
        if self is CredentialType.BASIC:
            return basic()
        if self is CredentialType.SECRET:
            return secret()
        if self is CredentialType.O_AUTH_CLIENT:
            return o_auth_client()
        if self is CredentialType.BRIDGE:
            return bridge()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .elasticsearch_auth_options import ElasticsearchAuthOptions
from .elasticsearch_credential import ElasticsearchCredential

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SiemElasticsearch(pydantic.BaseModel):
    """
    Configuration for Elasticsearch search and analytics engine. Supports both managed and self-hosted Elasticsearch deployments
    """

    auth_options: typing.Optional[ElasticsearchAuthOptions] = None
    create_index: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional. The index or data stream to use when writing events. Defaults to the 'index' setting if not set.
    """

    credential: ElasticsearchCredential
    index: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional. The index, data stream, or index alias to read events from.
    """

    skip_tls_verify: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, skips verification of the Elasticsearch server's TLS certificate.
    """

    url: str = pydantic.Field()
    """
    URL for the Elasticsearch API. This should be the base URL for the API, without any path components and must be HTTPS. For example, "https://tenant.elastic.com".
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .agent_type_id import AgentTypeId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Agent(pydantic.BaseModel):
    """
    An Agent (also known as a Sensor) is typically installed on an Operating System (OS) and serves as a specialized software component that can be designed to monitor, detect, collect, archive, or take action. These activities and possible actions are defined by the upstream system controlling the Agent and its intended purpose. For instance, an Agent can include Endpoint Detection & Response (EDR) agents, backup/disaster recovery sensors, Application Performance Monitoring or profiling sensors, and similar software.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the agent or sensor. For example: <code>AWS SSM Agent</code>.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The normalized caption of the type_id value for the agent or sensor. In the case of 'Other' or 'Unknown', it is defined by the event source.
    """

    type_id: typing.Optional[AgentTypeId] = pydantic.Field(default=None)
    """
    The normalized representation of an agent or sensor. E.g., EDR, vulnerability management, APM, backup & recovery, etc.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The UID of the agent or sensor, sometimes known as a Sensor ID or <code>aid</code>.
    """

    uid_alt: typing.Optional[str] = pydantic.Field(default=None)
    """
    An alternative or contextual identifier for the agent or sensor, such as a configuration, organization, or license UID.
    """

    vendor_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The company or author who created the agent or sensor. For example: <code>Crowdstrike</code>.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The semantic version of the agent or sensor, e.g., <code>7.101.50.0</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .attachment import Attachment
from .attachment_id import AttachmentId
from .attachment_metadata import AttachmentMetadata
from .comment import Comment
from .comment_id import CommentId
from .create_attachment_request import CreateAttachmentRequest
from .create_attachment_response import CreateAttachmentResponse
from .create_comment_request import CreateCommentRequest
from .create_comment_response import CreateCommentResponse
from .create_ticket_request import CreateTicketRequest
from .create_ticket_response import CreateTicketResponse
from .download_attachment_response import DownloadAttachmentResponse
from .field_mapping_id import FieldMappingId
from .get_ticket_response import GetTicketResponse
from .issue_type_id import IssueTypeId
from .list_attachments_metadata_response import ListAttachmentsMetadataResponse
from .list_comments_response import ListCommentsResponse
from .list_projects_response import ListProjectsResponse
from .list_remote_fields_response import ListRemoteFieldsResponse
from .patch_ticket_response import PatchTicketResponse
from .priority import Priority
from .project import Project
from .project_id import ProjectId
from .query_tickets_response import QueryTicketsResponse
from .remote_field import RemoteField
from .remote_field_schema import RemoteFieldSchema
from .remote_field_scope import RemoteFieldScope
from .remote_field_type_id import RemoteFieldTypeId
from .status import Status
from .ticket import Ticket
from .ticket_id import TicketId
from .value_mapping_id import ValueMappingId

__all__ = [
    "Attachment",
    "AttachmentId",
    "AttachmentMetadata",
    "Comment",
    "CommentId",
    "CreateAttachmentRequest",
    "CreateAttachmentResponse",
    "CreateCommentRequest",
    "CreateCommentResponse",
    "CreateTicketRequest",
    "CreateTicketResponse",
    "DownloadAttachmentResponse",
    "FieldMappingId",
    "GetTicketResponse",
    "IssueTypeId",
    "ListAttachmentsMetadataResponse",
    "ListCommentsResponse",
    "ListProjectsResponse",
    "ListRemoteFieldsResponse",
    "PatchTicketResponse",
    "Priority",
    "Project",
    "ProjectId",
    "QueryTicketsResponse",
    "RemoteField",
    "RemoteFieldSchema",
    "RemoteFieldScope",
    "RemoteFieldTypeId",
    "Status",
    "Ticket",
    "TicketId",
    "ValueMappingId",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AuditActions(str, enum.Enum):
    LIST = "list"
    ALL = "*"

    def visit(self, list_: typing.Callable[[], T_Result], all_: typing.Callable[[], T_Result]) -> T_Result:
        if self is AuditActions.LIST:
            return list_()
        if self is AuditActions.ALL:
            return all_()



# This file was auto-generated by Fern from our API Definition.

from .connector import Connector
from .connector_or_id import ConnectorOrId
from .filter_operation import FilterOperation
from .filter_type import FilterType
from .get_provider_capabilities_expand_options import GetProviderCapabilitiesExpandOptions
from .list_connector_capabilities_expand_options import ListConnectorCapabilitiesExpandOptions
from .list_connectors_capabilities_response import ListConnectorsCapabilitiesResponse
from .list_provider_capabilities_expand_options import ListProviderCapabilitiesExpandOptions
from .list_providers_capabilities_response import ListProvidersCapabilitiesResponse
from .provider_capabilities import ProviderCapabilities
from .provider_capabilities_response import ProviderCapabilitiesResponse
from .provider_filter import ProviderFilter
from .provider_operations import ProviderOperations
from .request_body import RequestBody

__all__ = [
    "Connector",
    "ConnectorOrId",
    "FilterOperation",
    "FilterType",
    "GetProviderCapabilitiesExpandOptions",
    "ListConnectorCapabilitiesExpandOptions",
    "ListConnectorsCapabilitiesResponse",
    "ListProviderCapabilitiesExpandOptions",
    "ListProvidersCapabilitiesResponse",
    "ProviderCapabilities",
    "ProviderCapabilitiesResponse",
    "ProviderFilter",
    "ProviderOperations",
    "RequestBody",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.timestamp import Timestamp
from ...objects.types.analytic import Analytic
from ...objects.types.api import Api
from ...objects.types.attack import Attack
from ...objects.types.cis_csc import CisCsc
from ...objects.types.cloud import Cloud
from ...objects.types.compliance import Compliance
from ...objects.types.enrichment import Enrichment
from ...objects.types.finding import Finding
from ...objects.types.kill_chain_phase import KillChainPhase
from ...objects.types.malware import Malware
from ...objects.types.metadata import Metadata
from ...objects.types.object import Object
from ...objects.types.observable import Observable
from ...objects.types.process import Process
from ...objects.types.resource_details import ResourceDetails
from ...objects.types.vulnerability import Vulnerability
from .activity_id import ActivityId
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .confidence_id import ConfidenceId
from .impact_id import ImpactId
from .risk_level_id import RiskLevelId
from .severity_id import SeverityId
from .state_id import StateId
from .status_id import StatusId
from .type_uid import TypeUid

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SecurityFinding(pydantic.BaseModel):
    """
    Security Finding events describe findings, detections, anomalies, alerts and/or actions performed by security products
    """

    activity_id: ActivityId = pydantic.Field()
    """
    The normalized identifier of the activity that triggered the event.
    """

    activity_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event activity name, as defined by the activity_id.
    """

    analytic: typing.Optional[Analytic] = pydantic.Field(default=None)
    """
    The analytic technique used to analyze and derive insights from the data or information that led to the finding or conclusion.
    """

    api: typing.Optional[Api] = pydantic.Field(default=None)
    """
    Describes details about a typical API (Application Programming Interface) call.
    """

    attacks: typing.Optional[typing.List[Attack]] = pydantic.Field(default=None)
    """
    The attack object describes the technique and associated tactics as defined by <a target='_blank' href='https://attack.mitre.org/wiki/ATT&CK_Matrix'>ATT&CK Matrix<sup>TM</sup></a>.
    """

    category_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event category name, as defined by category_uid value: <code>Findings</code>.
    """

    category_uid: CategoryUid = pydantic.Field()
    """
    The category unique identifier of the event.
    """

    cis_csc: typing.Optional[typing.List[CisCsc]] = pydantic.Field(default=None)
    """
    The CIS Critical Security Controls is a list of top 20 actions and practices an organization’s security team can take on such that cyber attacks or malware, are minimized and prevented.
    """

    class_uid: ClassUid = pydantic.Field()
    """
    The unique identifier of a class. A Class describes the attributes available in an event.
    """

    cloud: typing.Optional[Cloud] = pydantic.Field(default=None)
    """
    Describes details about the Cloud environment where the event was originally created or logged.
    """

    compliance: typing.Optional[Compliance] = pydantic.Field(default=None)
    """
    The compliance object provides context to compliance findings (e.g., a check against a specific regulatory or best practice framework such as CIS, NIST etc.) and contains compliance related details.
    """

    confidence: typing.Optional[str] = pydantic.Field(default=None)
    """
    The confidence, normalized to the caption of the confidence_id value. In the case of 'Other', it is defined by the event source.
    """

    confidence_id: typing.Optional[ConfidenceId] = pydantic.Field(default=None)
    """
    The normalized confidence refers to the accuracy of the rule that created the finding. A rule with a low confidence means that the finding scope is wide and may create finding reports that may not be malicious in nature.
    """

    confidence_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The confidence score as reported by the event source.
    """

    count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of times that events in the same logical group occurred during the event <strong>Start Time</strong> to <strong>End Time</strong> period.
    """

    custom_fields: typing.Optional[Object] = pydantic.Field(default=None)
    """
    A list of custom fields
    """

    data_sources: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of data sources utilized in generation of the finding.
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    The event duration or aggregate time, the amount of time the event covers from <code>start_time</code> to <code>end_time</code> in milliseconds.
    """

    end_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The end time of a time period, or the time of the most recent event included in the aggregate event.
    """

    end_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The end time of a time period, or the time of the most recent event included in the aggregate event.
    """

    enrichments: typing.Optional[typing.List[Enrichment]] = pydantic.Field(default=None)
    """
    The additional information from an external data source, which is associated with the event or a finding. For example add location information for the IP address in the DNS answers:</p><code>[{"name": "answers.ip", "value": "92.24.47.250", "type": "location", "data": {"city": "Socotra", "continent": "Asia", "coordinates": [-25.4153, 17.0743], "country": "YE", "desc": "Yemen"}}]</code>
    """

    evidence: typing.Optional[typing.Any] = pydantic.Field(default=None)
    """
    The data the finding exposes to the analyst.
    """

    finding: Finding = pydantic.Field()
    """
    The Finding object provides details about a finding/detection generated by a security tool.
    """

    impact: typing.Optional[str] = pydantic.Field(default=None)
    """
    The impact , normalized to the caption of the impact_id value. In the case of 'Other', it is defined by the event source.
    """

    impact_id: typing.Optional[ImpactId] = pydantic.Field(default=None)
    """
    The normalized impact of the finding.
    """

    impact_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The impact of the finding, valid range 0-100.
    """

    kill_chain: typing.Optional[typing.List[KillChainPhase]] = pydantic.Field(default=None)
    """
    The <a target='_blank' href='https://www.lockheedmartin.com/en-us/capabilities/cyber/cyber-kill-chain.html'>Cyber Kill Chain®</a> provides a detailed description of each phase and its associated activities within the broader context of a cyber attack.
    """

    malware: typing.Optional[typing.List[Malware]] = pydantic.Field(default=None)
    """
    A list of Malware objects, describing details about the identified malware.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the event/finding, as defined by the source.
    """

    metadata: Metadata = pydantic.Field()
    """
    The metadata associated with the event or a finding.
    """

    nist: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The NIST Cybersecurity Framework recommendations for managing the cybersecurity risk.
    """

    observables: typing.Optional[typing.List[Observable]] = pydantic.Field(default=None)
    """
    The observables associated with the event or a finding.
    """

    process: typing.Optional[Process] = pydantic.Field(default=None)
    """
    The process object.
    """

    raw_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    The raw event/finding data as received from the source.
    """

    resources: typing.Optional[typing.List[ResourceDetails]] = pydantic.Field(default=None)
    """
    Describes details about resources that were affected by the activity/event.
    """

    risk_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    The risk level, normalized to the caption of the risk_level_id value. In the case of 'Other', it is defined by the event source.
    """

    risk_level_id: typing.Optional[RiskLevelId] = pydantic.Field(default=None)
    """
    The normalized risk level id.
    """

    risk_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The risk score as reported by the event source.
    """

    severity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding severity, normalized to the caption of the severity_id value. In the case of 'Other', it is defined by the source.
    """

    severity_id: SeverityId = pydantic.Field()
    """
    <p>The normalized identifier of the event/finding severity.</p>The normalized severity is a measurement the effort and expense required to manage and resolve an event or incident. Smaller numerical values represent lower impact events, and larger numerical values represent higher impact events.
    """

    start_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The start time of a time period, or the time of the least recent event included in the aggregate event.
    """

    start_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The start time of a time period, or the time of the least recent event included in the aggregate event.
    """

    state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The normalized state of a security finding.
    """

    state_id: StateId = pydantic.Field()
    """
    The normalized state identifier of a security finding.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status, normalized to the caption of the status_id value. In the case of 'Other', it is defined by the event source.
    """

    status_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event status code, as reported by the event source.<br /><br />For example, in a Windows Failed Authentication event, this would be the value of 'Failure Code', e.g. 0x18.
    """

    status_detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status details contains additional information about the event/finding outcome.
    """

    status_id: typing.Optional[StatusId] = pydantic.Field(default=None)
    """
    The normalized identifier of the event status.
    """

    time: Timestamp = pydantic.Field()
    """
    The normalized event occurrence time or the finding creation time.
    """

    time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The normalized event occurrence time or the finding creation time.
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of minutes that the reported event <code>time</code> is ahead or behind UTC, in the range -1,080 to +1,080.
    """

    type_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event/finding type name, as defined by the type_uid.
    """

    type_uid: TypeUid = pydantic.Field()
    """
    The event/finding type ID. It identifies the event's semantics and structure. The value is calculated by the logging system as: <code>class_uid \* 100 + activity_id</code>.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the event schema. The names and values of those attributes are specific to the event source.
    """

    vulnerabilities: typing.Optional[typing.List[Vulnerability]] = pydantic.Field(default=None)
    """
    This object describes vulnerabilities reported in a security finding.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .create_device_request import CreateDeviceRequest
from .create_device_response import CreateDeviceResponse
from .device import Device
from .query_devices_response import QueryDevicesResponse

__all__ = ["CreateDeviceRequest", "CreateDeviceResponse", "Device", "QueryDevicesResponse"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...common.types.base import Base
from ...token_base.types.token_id import TokenId
from .organization_id import OrganizationId
from .organization_options import OrganizationOptions
from .organization_type import OrganizationType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Organization(Base):
    id: OrganizationId
    refresh_token_id: TokenId = pydantic.Field()
    """
    Organization refresh token id
    """

    organization_type: OrganizationType = pydantic.Field()
    """
    Organization type: root or standard
    """

    fullname: str = pydantic.Field()
    """
    Human friendly display name for this Organization
    """

    contact: typing.Optional[str] = pydantic.Field(default=None)
    """
    Organization email address
    """

    reply_to: typing.Optional[str] = pydantic.Field(default=None)
    """
    Reply-to email address, used for SMTP emails. Defaults to no-reply@synqly.com
    """

    picture: typing.Optional[str] = pydantic.Field(default=None)
    """
    Picture URL of the organization
    """

    options: typing.Optional[OrganizationOptions] = pydantic.Field(default=None)
    """
    Organization options
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

from ...capabilities_base.types.category_id import CategoryId
from .connector import Connector

ConnectorOrId = typing.Union[CategoryId, Connector]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...account_base.types.account_id import AccountId
from ...integration_base.types.integration_id import IntegrationId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class StatusEvent(pydantic.BaseModel):
    """
    Status event object
    """

    account_id: AccountId = pydantic.Field()
    """
    Account owner
    """

    integration_id: IntegrationId = pydantic.Field()
    """
    Integration object
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Time created
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message
    """

    request: int = pydantic.Field()
    """
    Request number
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from ...ocsf.resources.v_1_3_0.resources.inventoryinfo.resources.classes.types.inventory_info import InventoryInfo

Device = InventoryInfo



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.detectionfinding.resources.classes.types.detection_finding import (
    DetectionFinding as resources_ocsf_resources_v_1_3_0_resources_detectionfinding_resources_classes_types_detection_finding_DetectionFinding,
)

DetectionFinding = resources_ocsf_resources_v_1_3_0_resources_detectionfinding_resources_classes_types_detection_finding_DetectionFinding



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Account,
    CreateAccountRequest,
    CreateAccountResponse,
    CreateAccountResponseResult,
    GetAccountResponse,
    ListAccountsResponse,
    PatchAccountResponse,
    UpdateAccountRequest,
    UpdateAccountResponse,
)

__all__ = [
    "Account",
    "CreateAccountRequest",
    "CreateAccountResponse",
    "CreateAccountResponseResult",
    "GetAccountResponse",
    "ListAccountsResponse",
    "PatchAccountResponse",
    "UpdateAccountRequest",
    "UpdateAccountResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    GetMemberResponse,
    ListMembersResponse,
    PatchMemberResponse,
    UpdateMemberRequest,
    UpdateMemberResponse,
)

__all__ = [
    "GetMemberResponse",
    "ListMembersResponse",
    "PatchMemberResponse",
    "UpdateMemberRequest",
    "UpdateMemberResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...common.types.base import Base
from ...common.types.id import Id
from ...token_base.types.token_id import TokenId
from ...token_base.types.token_owner_type import TokenOwnerType
from ...token_base.types.token_pair import TokenPair

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class RefreshToken(Base):
    id: TokenId
    member_id: typing.Optional[Id] = pydantic.Field(default=None)
    """
    Member Id
    """

    owner_id: Id = pydantic.Field()
    """
    ID of the entity that owns this token
    """

    owner_type: TokenOwnerType = pydantic.Field()
    """
    Type of the entity that owns this token
    """

    expires: dt.datetime = pydantic.Field()
    """
    Time when this token expires and can no longer be used again.
    """

    token_ttl: str = pydantic.Field()
    """
    Token time-to-live
    """

    primary: TokenPair = pydantic.Field()
    """
    Primary running access and refresh tokens
    """

    secondary: typing.Optional[TokenPair] = pydantic.Field(default=None)
    """
    Temporary secondary TokenPair created after a RefreshToken operation
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class NetworkTraffic(pydantic.BaseModel):
    """
    The Network Traffic object describes characteristics of network traffic. Network traffic refers to data moving across a network at a given point of time. Defined by D3FEND <a target='_blank' href='https://d3fend.mitre.org/dao/artifact/d3f:NetworkTraffic/'>d3f:NetworkTraffic</a>.
    """

    bytes: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of bytes (in and out).
    """

    bytes_in: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of bytes sent from the destination to the source.
    """

    bytes_out: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of bytes sent from the source to the destination.
    """

    chunks: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of chunks (in and out).
    """

    chunks_in: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of chunks sent from the destination to the source.
    """

    chunks_out: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of chunks sent from the source to the destination.
    """

    packets: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of packets (in and out).
    """

    packets_in: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of packets sent from the destination to the source.
    """

    packets_out: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of packets sent from the source to the destination.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .create_notification_request import CreateNotificationRequest
from .create_notification_response import CreateNotificationResponse
from .get_notification_response import GetNotificationResponse
from .notification import Notification
from .notification_id import NotificationId
from .notification_status import NotificationStatus

__all__ = [
    "CreateNotificationRequest",
    "CreateNotificationResponse",
    "GetNotificationResponse",
    "Notification",
    "NotificationId",
    "NotificationStatus",
]



# This file was auto-generated by Fern from our API Definition.

from .types import Audit, AuditType, HttpMethod, ListAuditEventsResponse

__all__ = ["Audit", "AuditType", "HttpMethod", "ListAuditEventsResponse"]
